import java.awt.Color;
import java.io.Serializable;

import cubeengine.Cord3d;
import cubeengine.Sun;

/**
 * Ammunition. r ett skott som flyger genom rummet.
 * @author Per-Erik Kristensson
 */
public class Ammo extends Sun implements Serializable
{
    /**
     * Skapar en Ammo
     * @param sender Robot som skt
     * @param color  Frg p kulan
     */
	public Ammo(Robot sender, Color color)
	{
	    super(0, 0, 0, color);
	      
		//setTransparency(0.5f);
	    setRadius(0.07);  	
	    setGamma(4.0);
			
	    m_shooterId = sender.getId();
		double[][] matrix = Cord3d.createRotateMatrix(sender.getAngleX(), sender.getAngleY(), -sender.getAngleZ()*0);	
	
		double dx=0, dy=0, dz=-Ammo.AMMO_SPEED;
	
		m_vx = dx * matrix[0][0] + dy*matrix[0][1] + dz*matrix[0][2];
		m_vy = dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2];
		m_vz = dx * matrix[2][0] + dy*matrix[2][1] + dz*matrix[2][2];
	
	    dz =-sender.getRadius();

		//Stt p samma position som sndaren + radien
		double  x = sender.getCenter().getX() + dx * matrix[0][0] + dy*matrix[0][1] + dz*matrix[0][2], 
				y = sender.getCenter().getY() + dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2], 
				z = sender.getCenter().getZ() + dx * matrix[2][0] + dy*matrix[2][1] + dz*matrix[2][2];
			        
		        getCenter().set(x, y, z);
    }
    
	/**
	 * Hmtar robotid p som skt kulan
	 * @return robotenid p den som skt
	 */
	public int getShooterID()
	{
		return m_shooterId;
	}

	/**
	 * Hmtar kulans frg
	 * @return kulans frg
	 */
	public Color getColor()
	{
		return m_color;
	}

	/**
	 * Flyttar kulan
	 * @return false om kulan inte ska flyttas mer
	 */
	public boolean tick()
	{
		if(	(getCenter().getX() + m_vx < GameEngine.XMIN && m_vx < 0) ||
			(getCenter().getX() + m_vx > GameEngine.XMAX && m_vx > 0) )
		{
			m_vx = -m_vx;
			return false;
		}

		if(	(getCenter().getZ() + m_vz < GameEngine.ZMIN && m_vz < 0) ||
			(getCenter().getZ() + m_vz > GameEngine.ZMAX && m_vz > 0) )
		{
			m_vz = -m_vz;
			return false;
		}



		if( getCenter().getY() + m_vy + getRadius() < GameEngine.YMIN && m_vy < 0)
		{
			m_vy = -m_vy;
			return false;
		}

		getCenter().add(m_vx, m_vy, m_vz);

		m_age++;

		if(m_age > AMMO_LIFE_TIME)
			return false;

		int r = m_color.getRed()-1;	if(r<0) r=0;
		int g = m_color.getGreen()-3;	if(g<0) g=0;
		int b = m_color.getBlue()-4;	if(b<0) b=0;

		m_color = new Color(r,g,b);

		return true;
	}
	
	/**
	 * Retunerar hur mycket liv en robot ska mista
	 * om den blir trffat
	 * @return Hlsominskning
	 */
	public double getKillValue()
	{
		return  MAX_AMMO_KILL - 
				(MAX_AMMO_KILL-MIN_AMMO_KILL)*m_age/AMMO_LIFE_TIME;
	}

	/**
	 * Hastighet i olika dimensioner
	 */
	protected double m_vx, m_vy, m_vz;
	
	/**
	 * ldern p skottet (i tick)
	 */
	protected int m_age=0;
	
	/**
	 * ID-nummer p den som skt
	 */
	protected int m_shooterId=0;

	/**
	 * Minsta hlsofrlust vid trff.
	 */
    public static final double MIN_AMMO_KILL = 5.0;
    
    /**
     * Maximal hlsofrlust vid trff
     */
    public static final double MAX_AMMO_KILL = 50.0;
    
    /**
     * Maximal livslngd p ammuntion
     */
    public static final int AMMO_LIFE_TIME = 3*GameEngine.TPS;
    
    /**
     * Hastighet p ammunition
     */
    public static final double AMMO_SPEED = 3*14.0/GameEngine.TPS;
	


}
