import java.awt.Color;
import java.io.Serializable;

import cubeengine.Cord3d;
import cubeengine.Figure;

/**
 * En Box r en lda med liv, ammunition, oddlighet eller radar.
 *  
 * @author Per-Erik Kristensson
 */
class Box extends Figure implements Serializable
{
    /**
     * Skapar en lda p kordinat (x,y,z) med innehllet
     * definitera i type
     * @param x Kordinat 
     * @param y Kordinat 
     * @param z Kordinat
     * @param type Typ av innehll
     * @param lifeTime antal tick ldan ska leva
     */
	public Box(double x, double y, double z, int type, boolean isRandom, int lifeTime)
	{
		super("cube");
		
		m_tickCounter = lifeTime;
		
		rotateAxel(0, Math.PI/4, Math.PI/4);
		rescale(0.5, 0.5, 0.5);		
		
		m_type=type;
		m_isRandom = isRandom;

		setRadius( Math.sqrt( 3*0.5*0.5 ) );
		//	setTransparency( 0.6f );
		m_rotateMatrix = Cord3d.createRotateMatrix( Math.PI*2 *0.45/ GameEngine.TPS, 0, 0 );

		//ndra inte sidfrgen om ldan r slumpmssig
		if(!m_isRandom)
		{
			if(m_type == Box.HEALTH50)
				setAllSideColor( new Color( 212, 34, 133 ) );
			else if(m_type == Box.IMMORTAL)
			{
				load("pyramid");
				setAllSideColor( new Color( 4, 55, 208 ) );
			}
			else if(m_type == Box.AMMO)
				setAllSideColor( new Color( 200, 200, 123) );
			else if(m_type == Box.RADAR )
				setAllSideColor( new Color( 43, 232, 55 ) );
		}
		getCenter().set(x,y,z);
		
	}
	
	/**
	 * Retunerar en slumpmssig ldtyp
	 * @return en slumpmssig ldtyp
	 */	
	public static int getRandomType()
	{
		int type = Box.HEALTH50; 

		double rand = Math.random();
		
		if(rand > 0.85)
			type = Box.RADAR;
		else if(rand > 0.80)
			type = Box.IMMORTAL;
		else if(rand > 0.4)
			type = Box.AMMO;

		return type;
	}
	
	/**
	 * Kollar hur mnga tick ldan har kvar att leva
	 * @return tick kvar att leva
	 */
	public int getLifeLeft()
	{
	    return m_tickCounter;
	}
	/**
	 * Tickar ldan
	 * @return true om ldan ska sluta existera
	 */
	public boolean tick()
	{
	    m_tickCounter--;
	    if(m_tickCounter <= 0)
	        return true;
	    		
		rotateAxel(m_rotateMatrix);
		
		return false;
	}
	
	/**
	 * Kollar om ldan blir trffad
	 * @param am Ammunition
	 * @return true om ldan blir trffad
	 */
	public boolean hitTest(Ammo am)
	{
		return isInside(am);
	}
	
	/**
	 * Hmtar ldtyp
	 * @return Ldtyp
	 */
	public int getType()
	{
		return m_type;
	}
	
	/**
	 * Kollar om ldan visas som slumpmssig
	 * @return true om ldan visas som slumpmssig.
	 */
	public boolean isRandom()
	{
	    return m_isRandom;
	}
	
	/**
	 * Rotationsmatris (anvnds fr att vrida ldan)
	 */
	protected double [][] m_rotateMatrix=null;
	
	/**
	 * Antalet tick kvar ldan har att leva
	 */
	protected int m_tickCounter=0; 
	
	/**
	 * Typ av innehll i ldan
	 */
	protected int m_type;
	
	/**
	 * Om ldan r slumpmssig
	 */
	protected boolean m_isRandom;
	
	/**
	 * ID fr hlsolda
	 */
	public static final int HEALTH50=0;
	
	/**
	 * ID fr radarlda
	 */
	public static final int RADAR=1;
	
	/**
	 * ID fr oddlighetslda
	 */
	public static final int IMMORTAL=2;
	
	/**
	 * ID fr ammunitionslda
	 */
	public static final int AMMO=3;
	
	/**
	 * Livslngd p en lda
	 */
    public static final int BOX_LIFETIME = 7*GameEngine.TPS;
    
    /**
     * Hur mycket man fr i liv nr man tar en hlsolda
     */
    public static final double BOX_HEALTH = 50; 
    
    /**
     * Hur mycket ammuntion i en ammuntionslda.
     */
	public static final int BOX_AMMO_COUNT = 10;

}
