import java.awt.Color;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.ArrayList;

/**
 * En Client skter kommuniktiation p klienten till servern,
 * eller p servern till klient.
 * 
 * @author Per-Erik Kristensson
 *
 */
public class Client extends Thread
{
    /**
     * Skapar en client som krs p servern, och skter
     * allts kommuniktation mellan servern och klienten.
     * 
     * @param socket Socketen som klienten arbetar p
     */
	public Client(Socket socket)
	{
		try
		{			
			m_socket = socket;
			m_outStream = new ObjectOutputStream(m_socket.getOutputStream());	
		}
		catch(Exception e)
		{
			System.out.println("Create client failure: " + e.getMessage());
		}

		m_isServer = true;
	}
	
	/**
	 * Skapar en Client som krs p klienten. 
	 * 
	 * @param socket Socketen som klienten arbetar p
	 * @param name   Robotens namn
	 * @param color  Robotens frg
	 * @param type   Typ av robot
	 */
	public Client(Socket socket, String name, Color color, int type)
	{
		try
		{			
			m_socket = socket;
			//m_inStream = new ObjectInputStream(m_socket.getInputStream());
			m_outStream = new ObjectOutputStream(m_socket.getOutputStream());	
		}
		catch(Exception e)
		{
			System.out.println("Create client failure: " + e.getMessage());
		}
				
		m_isServer = false;

		//Skicka ivg vilket namn, frg med mera till servern.
		JobAddRobot info = new JobAddRobot(false, name, color, type);
		send(info);
	}

	/**
	 * run-trd. Krs s lnge klienten r uppkopplad.
	 */
	public void run()
	{
		while(true)
		{
			try
			{
				if(m_inStream == null)
					m_inStream = new ObjectInputStream(m_socket.getInputStream());

				Job p = (Job) m_inStream.readObject();

				if(p instanceof JobStartGame)
				{
				    JobStartGame start = (JobStartGame) p;
				    m_engine.m_robots = start.robots;
				    m_engine.m_ammos = start.ammos;
				    m_engine.m_boxes = start.boxes;
				    
				    for(int i=0; i<m_engine.m_robots.size(); i++)
				    {
				        m_engine.getRobot(i).setEngine(m_engine);
				        m_engine.m_3dEngine.add( m_engine.getRobot(i) );
				        m_engine.getRobot(i).setAi(false);
				    }
				    
				    for(int i=0; i<m_engine.m_ammos.size(); i++)
				    {
				        m_engine.m_3dEngine.add( (Ammo) m_engine.m_ammos.get(i) );
				    }

				    for(int i=0; i<m_engine.m_boxes.size(); i++)
				    {
				        m_engine.m_3dEngine.add( (Box) m_engine.m_boxes.get(i) );
				    }
				    
					//Stll in lite saker
					m_robotId = ((JobStartGame)p).m_robotId;
					m_engine.setUserRobot( m_engine.getRobot(m_robotId) );
					
					System.out.println("Jag r " + m_engine.getUserRobot().getName());

					//Meddela alla lyssnare
					for(int i=0; i<m_listeners.size(); i++)
					{
						((ClientListener) m_listeners.get(i)).startGame(this);
					}

					System.out.println("Spel startat");
					
					continue;
				}
				//Tick kommer nr klienten ska ticka fram
				else if(p instanceof JobTick)
				{
					m_engine.tick();
				}
				else
				if( !(m_isServer && p instanceof JobAddRobot))    
				{
				    //Fr vriga jobb, lgg till dessa i motorn
			        m_engine.addJobFromServer(p);
				}

				//Meddela alla lyssnare att nytt jobb har kommit 
				for(int i=0; i<m_listeners.size(); i++)
				{
					((ClientListener) m_listeners.get(i)).jobRecieved(p, this);
				}
				
			}
			catch( Exception e)
			{
				System.out.println("Client::run failure: " + e.getMessage());
				//e.printStackTrace();
				
				close();

				return;
			}
		}
	}
	
	/**
	 * Skickar ivg ett Job till mottagaren
	 * @param p Job att skicka
	 */
	public void send(Job p)
	{
		try
		{
			m_outStream.writeObject(p);
		}
		catch(Exception e)
		{
			System.out.println("Client::send failure: " + e.getMessage());
			e.printStackTrace();

		}
	}

	/**
	 * Ser till att utstrmsbufferten skickas
	 */
	public void flush()
	{
		try
		{
			m_outStream.flush();
		}
		catch(Exception e)
		{
			System.out.println("Client::flush failure: " + e.getMessage());
		}
	}
	
	/**
	 * Stnger socketen
	 */
	public void close()
	{
		try
		{
			if(!m_isServer)
				m_engine.makeLocalGame();
			
			m_socket.close();
		}
		catch(Exception e)
		{
		}

		for(int i=0; i<m_listeners.size(); i++)
		{
			((ClientListener) m_listeners.get(i)).clientDisconnected(	this);
		}
	}

	/**
	 * Lgger till en clientlyssnare
	 * @param cl ClientListener att lgga till
	 */
	public void addClientListener(ClientListener cl)
	{
		m_listeners.add(cl);
	}
	
	/**
	 * Tar bort till en clientlyssnare
	 * @param cl Att tas bort
	 */
	public void removeClientListener(ClientListener cl)
	{
		m_listeners.remove(cl);
	}

	/**
	 * Stter RobotID.
	 * @param i Nytt ID
	 */
	public void setRobotId(int i)
	{
		m_robotId=i;
	}

	/**
	 * Hmtar RobotID
	 * @return robotid
	 */
	public int getRobotId()
	{
		return m_robotId;
	}

	/**
	 * Stter spelmotor att anvnda
	 * @param e Spelmotor
	 */
	public void setEngine(GameEngine e)
	{
		m_engine=e;
	}

	
	/**
	 * Socketen som kommunicerar med klienten/servern
	 */
	protected Socket m_socket=null;
	
	/**
	 * Instrm fr socketen.
	 */
	protected ObjectInputStream m_inStream=null;
	
	/**
	 * Utstrm fr socketen
	 */
	protected ObjectOutputStream m_outStream=null;
	
	/**
	 * Array med alla ClientLyssnare
	 */
	protected ArrayList m_listeners=new ArrayList();
	
	/**
	 * Vilken spelmotor som anvnds
	 */
	protected GameEngine m_engine=null;
	
	/**
	 * Om detta objekt krs p servern och kommunicerar med en klient
	 */
	protected boolean m_isServer=false;
	
	/**
	 * Vilket robotid denna klient styr
	 */
	protected int m_robotId=-1;
	

}