import cubeengine.Cord3d;
import cubeengine.Figure;
import cubeengine.Side;

/**
 * En FlyingSide r en flygande sida. En FlyingSide
 * skapas genom att kopiera en sida frn figur. Objektet
 * har en hastighet och flyttas och roteras fr varje 
 * gng tick anropas.
 * 
 * @author Per-Erik Kristensson
 */
public class FlyingSide extends Side
{
    /**
     * Skapar en flygande sida
     * @param cb Figur att kopiera frn
     * @param s  Vilken sida att kopiera
     * @param vx Hastighet i x-led
     * @param vy Hastighet i y-led
     * @param vz Hastighet y z-led
     */
	public FlyingSide(Figure cb, int s, double vx, double vy, double vz)
	{
		for(int i=0; i<cb.getSide(s).getCordCount(); i++)
		{
			//addCord( new Cord3d(  cb.getSide(s).getCord(i) ) );
			addCord( (Cord3d) cb.getSide(s).getCord(i).clone() );
		}
		
		calcMaxCornerDiffLength();


		m_color = cb.getSide(s).getColor();
		m_baseColor = cb.getSide(s).getBaseColor();
		m_borderColor = cb.getSide(s).getBorderColor();
		m_borderBaseColor = cb.getSide(s).getBorderBaseColor();

		cb.getSide(s).recalcCenter();

		m_vx=cb.getSide(s).getCenter().getX()/2;
		m_vy=cb.getSide(s).getCenter().getY()/2;
		m_vz=cb.getSide(s).getCenter().getZ()/2;

		recalcCenter();

		getCenter().add( cb.getCenter() );

		m_borderColor = null;
		setParent(null);

		//Rotera sidan p nt lite konstigt stt
		final double multi=2*Math.PI*1.2/GameEngine.TPS;
		m_matrix = Cord3d.createRotateMatrix(   (m_vx+m_vy+3*Math.random())*multi,
												(m_vy+m_vz+3*Math.random())*multi,
												(m_vz+m_vx+m_vy+3*Math.random()) *multi);
	}

	/**
	 * Flyttar sidan
	 * @return false om sidan ska sluta existera
	 */
	public boolean tick()
	{
	    m_age++;

		if(m_age > GameEngine.TPS)
			return false;

		getCenter().add(m_vx, m_vy, m_vz);

		rotateAxel(m_matrix);
		
		//Om sidan utanfr banan, retunera falskt
		{
			if(	(getCenter().getX() + m_vx < GameEngine.XMIN && m_vx < 0) ||
				(getCenter().getX() + m_vx > GameEngine.XMAX && m_vx > 0) )
			{
				//m_vx = -m_vx;
				return false;
			}

			if(	(getCenter().getZ() + m_vz < GameEngine.ZMIN && m_vz < 0) ||
				(getCenter().getZ() + m_vz > GameEngine.ZMAX && m_vz > 0) )
			{
				//m_vz = -m_vz;
				return false;
			}

			if( getCenter().getY() + m_vy + getRadius() > GameEngine.YMAX && m_vy > 0)
			{
				//m_vy = -m_vy;
				return false;
			}
		}

		//rescale(0.93, 0.93, 0.93);

		return true;		
	}

	/**
	 * Hastigheter i olika dimensioner
	 */
	protected double m_vx=0, m_vy=0, m_vz=0;
	
	/**
	 * Rotationsmatris (roterar sidan varje tick)
	 */
	protected double[][] m_matrix;
	
	/**
	 * lder p sidan
	 */
	protected int m_age=0;
}
