import java.awt.Color;
import java.net.Socket;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 * GameConnector r ett fnster som visar statusen nr man ansluter till ett spel.
 * @author Per-Erik Kristensson
 */
public class GameConnector extends JFrame implements ClientListener, Runnable
{
    /**
     * Skapar ett GameConnector-objekt
     * @param name	Namnet p roboten
     * @param color	Robotens frg
     * @param robotType	Robottyp
     * @param servername	Namnet p servern
     */
    public GameConnector(String name, Color color, int robotType, String servername)
	{
		super("Ntverksspel - Ansluter till " + servername);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		m_name=name;
		m_color=color;
		m_robotType=robotType;
		m_servername=servername;

		JPanel pane = new JPanel();

		m_status = new JLabel("Ansluter till " + servername + "... ");
		pane.add( m_status );

		setContentPane(pane);		
		pack();
		setVisible(true);

		Thread connector = new Thread(this);
		connector.start();
	}

	/**
	 * Krs i en trd fr att ansluta till servern.
	 */
	public void run()
	{
		m_engine.setNetMode(GameEngine.CLIENT);
			
		try
		{
			Socket sock =  new Socket(m_servername, 12345);
			m_client = new Client(sock, m_name, m_color, m_robotType);
			m_client.setEngine(m_engine);
			m_client.addClientListener(this);
			m_client.start();
		}
		catch(Exception e)
		{
			m_status.setText("Anslutningen misslyckades.");
				
			JOptionPane.showMessageDialog(null, "Det gick inte att ansluta till \"" + m_servername + "\"");
				
			System.exit(0);
		}

		m_status.setText("Initierar spel...");
	}

	public void jobRecieved(Job p, Client sender){};
	public void clientDisconnected(Client sender)
	{
		//Anslutningen har misslyckats
		m_status.setText("Initieringen misslyckades");
	};
	
	/**
	 * Startar spelet (Skickas frn Client nr initeringen r klar)
	 */
	public void startGame(Client sender)
	{		
		m_engine.setClient(sender);

		GameFrame game = new GameFrame(m_engine);

		setVisible(false);
		
		m_client.removeClientListener(this);
	}
	
	/**
	 * Spelmotor som ska anvndas
	 */
	protected GameEngine m_engine = new GameEngine();
	
	/**
	 * Client som ansluter till servern.
	 */
	protected Client m_client=null;
	
	/**
	 * Namnet p denna klients robot
	 */
	protected String m_name;
	
	/**
	 * Frg p denna klients robot
	 */
	protected Color m_color;
	
	/**
	 * Denna klients robottyp
	 */
	protected int m_robotType;
	
	/**
	 * Namnet p servern
	 */
	protected String m_servername;
	
	/**
	 * Statuslabel
	 */
	protected JLabel m_status;


}