import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.PixelGrabber;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Timer;
import java.util.TimerTask;
import java.net.URL;

import cubeengine.Cord3d;
import cubeengine.CubeEngine;
import cubeengine.Figure;
import cubeengine.Side;
import cubeengine.Sun;
import java.awt.image.*;
import java.awt.*;
import java.awt.image.*;
import javax.swing.*;


/**
 * GameEngine r sjlva spelmotorn av spelet. Den har hand om alla robotar,
 * flygande skott, sidor med mera.
 * 
 * Alla rrelser med mera sparas som job i motorn. Robotar anropar t ex
 * robotMove nr de vill flytta p sig, d lggs ett Job till. Robotar
 * flyttar allts inte p sig sjlva direkt, utan alla rrelser mste
 * g via spelmotorn.
 * 
 * TPS gnger per sekund anropas funktionen tick(), denna utfr alla jobb
 * i den ordning som de r lagrade. Om motorn r servern i ett spel skickas
 * frst alla jobb till alla klienter s att de vet vad som ska gras. P klienterna
 * sparas allts inte ngra job, utan nr en rrelse ska ske dr skickas jobbet
 * till servern som sedan skickar tillbaka det med alla de andra jobben.
 * 
 * Spelmotorn hller ocks koll p vilken Robot som anvndaren har. Denna kan
 * hmtas med getUserRobot()
 * 
 */
public class GameEngine 
{
	/**
	 * Skapar en GameEngine.
	 */
	public GameEngine()
	{		
		createRoom();
	}

	/**
	 * Gr s att de nuvarande spelet krs som ett lokalt spel.
	 */
	public void makeLocalGame()
	{
		System.out.println("Kr som lokalt spel");
		m_client = null;
		m_server = null;
		m_netMode = LOCAL;
		start();
		
		for(int i=0; i<getNumberOfRobots(); i++)
			if(getRobot(i) != getUserRobot())
				getRobot(i).setAi(true);
	}
	
	protected class EngineTimerTask extends TimerTask 
	{
		protected EngineTimerTask(GameEngine e)
		{
			m_engine=e;
		}

		synchronized public void run() 
		{
			m_engine.tick();
		}
	    
		protected GameEngine m_engine=null;
	}

	/**
	 * Startar spelmotorn s att den brjat ticka.
	 */
	synchronized public void start()
	{
		if(m_netMode == GameEngine.CLIENT)
			return;

		if(m_timer == null)
		{
		    EngineTimerTask enginetask = new EngineTimerTask(this);
		    
		    m_timer = new Timer();
		    m_timer.schedule(enginetask, 0, 1000/GameEngine.TPS);
		}
	}

	/**
	 * Stannar spelmotorn s att den slutar att ticka.
	 */
	synchronized public void stop()
	{	    
	    m_timer.cancel();
	    m_timer=null;
	}

	/**
	 * Lgger till en GameEngineListener.
	 */
	public void addGameEngineListener(GameEngineListener listener)
	{
		m_listeners.add(listener);
	}

	/**
	 * Tar bort till en GameEngineListener.
	 */
	public void removeListener(GameEngineListener listener)
	{
		m_listeners.remove(listener);
	}
	
	/**
	 * Anropar alla lyssnare
	 */
	protected void noteListeners()
	{
		for(int i=0; i<m_listeners.size(); i++)
			( (GameEngineListener) m_listeners.get(i) ).engineUpdated();
	}
	
	/**
	 * Kollar om en punkt i rummet r fri frn robotar.
	 * 
	 * @param x Kordinat x
	 * @param y Kordinat y
	 * @param z Kordinat z
	 * @param ignore Objekt som inte ska kollas. 
	 * @return true om punkten r ledig
	 */    	
	public boolean isFree(double x, double y, double z, Figure ignore)
	{
		double radius=0;
		if(ignore != null)
			radius=ignore.getRadius()*1.1;	//Lgg p 10% i marginalen
		
		if( x < GameEngine.XMIN+radius || x > GameEngine.XMAX-radius ||
			z < GameEngine.ZMIN+radius || z > GameEngine.ZMAX-radius )
				return false;
				
		for(int i=0; i<getNumberOfRobots(); i++)
		{
			Robot r = getRobot(i);
			
			if(r!=ignore && r.isAlive() && r.isInside( x, y, z ))
			{
				return false;
			}
		}
		
		return true;
	}

	/**
	 * Retunerar antalet ldor i rummet
	 * @return antalet ldor i rummet.
	 */
	public int getNumberOfBoxes()
	{
		return m_boxes.size();
	}
	
	/**
	 * Hmtar en lda
	 * @return lda nummer n
	 */
	public Box getBox(int n)
	{
		return (Box) m_boxes.get(n);
	}
	
    /**
	 * Retunerar antalet robotar i rummet
	 * @return antalet robotar i spelet
	 */	
	public int getNumberOfRobots()
	{
		return m_robots.size();
	
	}
	
	/**
	 * Hmtar den robot som anvndaren har.
	 * @return anvndarroboten.
	 */
	public Robot getUserRobot()
	{
		return m_robot;
	}

	/**
	 * Stter vilken robot anvndaren ska anvnda.
	 */
	public void setUserRobot(Robot r)
	{
		m_robot = r;
	}

	/**
	 * Lgger till ett jobb fr en robot att hoppa
	 */
	public void robotJump(Robot r)
	{
		addJob( new JobJump( r.getId() ) );
	}

	/**
	 * Lgger till ett jobb som gr att en robot skjuter
	 */
	public void robotShoot( Robot r )
	{
		addJob( new JobShoot( r.getId() ) );
	}

	/**
	 * Lgger till ett jobb som gr att en robot terupplivas
	 */
	public void robotRecarniate(Robot r)
	{
		addJob( new JobRecarniate( r.getId(), getRandomPosition(r) ) );
	}
	
	/**
	 * Lgger till ett jobb som gr att en robot flyttas
	 */
	public void robotMove(Robot r, double dx, double dy, double dz)
	{	
		addJob( new JobMove( r.getId(), dx, dy, dz ) );
	}

	/**
	 * Lgger till ett jobb som gr att en robot roterar
	 */
	public void robotRotate(Robot r, double dx, double dy, double dz)
	{
		addJob( new JobRotate( r.getId(), dx, dy, dz ) );
	}

	/**
	 * Lgg till en ldan ngonstans p planen
	 */
	public void addRandomBox()
	{
		Cord3d pos = getRandomPosition(null);
		pos.setY( 0.25*GameEngine.YMIN );
		addJob( new JobAddBox(	pos.getX(), 
		        				pos.getY(), 
		        				pos.getZ(), 
		        				Box.getRandomType(), 
		        				Math.random()>0.7,
		        				Box.BOX_LIFETIME
		        			)
		       );
	}

	/**
	 * Lgger till ett skott.
	 */
	public void addAmmo(Ammo am)
	{
		m_ammos.add(am);
		m_3dEngine.add(am);

	}

	/**
	 * Lgger till en robot med hjlp av ett jobb.
	 */
	public void addRobot(boolean ai)
	{
		if(ai && m_netMode == CLIENT)
		{
			System.out.println("Klienter kan inte lgga till robotar.");
			return;
		}
		
		addJob( new JobAddRobot(
			ai, 
			"Robot " + (int)( 100* Math.random() ),
			new Color(	(int)(255*Math.random()),
			(int)(255*Math.random()),
			(int)(255*Math.random())
			),
			(int)(Robot.ROBOT_COUNT*Math.random())
			)
			);
	}

	/**
	 * Lgger till en robot direkt i spelet, utan att g
	 * vgen via job.
	 */
	public void addRobotDirectly(Robot r)
	{
		System.out.println("Lgg till robot: " + r.getName());
		m_robots.add(r);
		m_3dEngine.add(r);
	}
	
	/**
	 * Retunerar en slumpmssig kordinat som r fri frn robotar.
	 */
	protected Cord3d getRandomPosition(Figure ignore)
	{
		double x = 	GameEngine.XMIN + Math.random()*(GameEngine.XMAX-GameEngine.XMIN),
			y = 	0, 
			z =	GameEngine.ZMIN + Math.random()*(GameEngine.ZMAX-GameEngine.ZMIN);
		
		//S att man inte hamnar i en vgg
		x *= 0.9;
		z *= 0.9;
		
		while(!isFree(x,y,z,null))
		{
			x = 	GameEngine.XMIN + Math.random()*(GameEngine.XMAX-GameEngine.XMIN);
			y = 	0; 
			z =		GameEngine.ZMIN + Math.random()*(GameEngine.ZMAX-GameEngine.ZMIN);
			
			x *= 0.9;
			z *= 0.9;
		}
		
		return new Cord3d(x,y,z);
	}

	/**
	 * Hmtar en robot
	 */
	public Robot getRobot(int r)
	{
		return (Robot) m_robots.get(r);
	}
	
	/**
	 * Lgger till en extra sol i grafikmotorn
	 */
	public void addExtraSun(Sun s)
	{
		m_3dEngine.add(s);
	}
	
	/**
	 * Raderar en sol frn grafikmotorn
	 */
	public void removeExtraSun(Sun s)
	{
		m_3dEngine.remove(s);
	}
	
	/**
	 * Hmtar den Robot som r efter Robot r
	 * @return Robot nummer r
	 */
	public Robot getNextRobot(Robot r)
	{
		int pos = r.getId(); 
		
		if(pos < 0)
			pos = 0;
		
		pos = (pos+1)%m_robots.size();
		
		return getRobot(pos);
			
	}
	
	/**
	 * Tickar fram spelet. Fljande grs:
	 *	
	 *	1. Alla jobb utfrs.
	 *	2. Om motorn r en server s skickas att jobb till klienterna
	 *  3. Om motorn r en server s lggs eventuellt en lda till.
	 *  4. Alla ldor roteras.
	 *  5. Alla skott flyttas.
	 *  6. Alla flygande sidor flyttas.
	 *  7. Robotar och ldor testar om de blir trffade.
	 *  8. Alla robotar tickar.
	 */
	public synchronized void tick()  
	{    
		if(m_netMode == SERVER)
			m_server.sendJobs(m_jobs);

		applyJobs();

		//Ej i klient
		if(m_netMode != CLIENT)
			if(m_boxes.size() < 10 && Math.random() < 5.0/GameEngine.TPS )
				addRandomBox();
		
		//Rotera ldor
		for(int b=m_boxes.size()-1; b>=0; b--)
		{
			if( ((Box) m_boxes.get(b)).tick() )
			{
				((Box) m_boxes.get(b)).setVisible(false);
				m_3dEngine.remove( m_boxes.get(b) );
				m_boxes.remove(b);
				continue;
			}
			
			Box box = (Box) m_boxes.get(b);		
			
			for(int c=0; c<m_robots.size(); c++)
			{
				Robot r = getRobot(c);

				if( r.boxHitTest(box) )
				{
					m_3dEngine.remove(box);
					m_boxes.remove(b);
					break;
				}				
			}
		}

		//Flytta ammunationen
		for(int a=m_ammos.size()-1; a>=0; a--)
		{
			if(! ((Ammo) m_ammos.get(a)).tick() )
			{
				m_3dEngine.remove( m_ammos.get(a) );
				m_ammos.remove(a);
			}
		}

		//Flytta flygande sidor
		for(int f=m_flyingSides.size()-1; f>=0; f--)
		{
			if(! ((FlyingSide) m_flyingSides.get(f)).tick() )
			{
				m_3dEngine.remove( m_flyingSides.get(f) );
				m_flyingSides.remove(f);
			}

		}

		//Trfftester
		for(int a=m_ammos.size()-1; a>=0; a--)
		{
			Ammo am = ((Ammo) m_ammos.get(a));

			//Testa om robotar blir trffade
			for(int c=0; c<m_robots.size(); c++)
			{
				Robot r = getRobot(c);

				int hit = r.hitTest(am);
				
				if(hit != Robot.NOHIT)
				{
					m_ammos.remove(am);
					m_3dEngine.remove(am);

					if(!r.isImmortal())
					{
						if(hit == Robot.KILLHIT)
						{
							createExplosion(r, am, r.getSideCount());
						}
						else
							createExplosion(r, am, 2);
					}

					break;
				}
			}
			
			//Testa om ldorna blir trffade
			for(int b=0; b<m_boxes.size(); b++)
			{
				Box box = (Box) m_boxes.get(b);

				if(box.hitTest(am))
				{
					createExplosion(box, am, box.getSideCount());
					
					m_ammos.remove(am);
					m_3dEngine.remove(am);
					
					m_boxes.remove(box);
					m_3dEngine.remove(box);
					
				}

			}
		}

		for(int r=0; r<m_robots.size(); r++)
		{
			getRobot(r).tick();			
		}
		
		noteListeners();
	}

	/**
	 * Skapar en explosion. Ett visst antal sidor frn en figur
	 * kopiera till flygande sidor.
	 */
	protected void createExplosion(Figure f, Ammo am, int sideCount)
	{
		for (int i = 0; i < sideCount; i++) //f.getSideCount(); i++) 
		{
			FlyingSide fs = new FlyingSide(f, i, 0.013, 0.014,
					0.015);

			fs.m_vx += am.m_vx;
			fs.m_vy += am.m_vy;
			fs.m_vz += am.m_vz;

			m_3dEngine.add(fs);
			m_flyingSides.add(fs);
		}
	}

	/**
	 * Ritar ut vrlden utifrn anvndarrobotens perspektiv
	 */
	public void render(Graphics2D g, int centerx, int centery)
	{
		render(g, centerx, centery, getUserRobot());
	}

	/**
	 * Ritar ut vrlden utifrn en robots perspektiv.
	 */
	public void render(Graphics2D g, int centerx, int centery, Robot camera)
	{
		m_3dEngine.draw( g, centerx, centery, camera );
		
		if(!camera.isAlive())
		{
			g.setColor(Color.green);
			g.drawLine(0, 0, 2*centerx, 2*centery);
			g.drawLine(0, 2*centery, 2*centerx, 0);
		}

		if(camera.isAlive() && camera.isImmortal())
			drawImmortalRing(g, centerx, centery);

		if(m_showScore || !camera.isAlive())
			drawScores(g, centerx, centery);
		
		if(camera.getRadarValue() > 0)
			drawRadar(g, centerx, centery);

		if(camera.getLastHitTick() < GameEngine.TPS)
			drawHit(g, centerx, centery, camera.getLastHitTick());
	}
	
	/**
	 * Bestmmer om pongen ska ritas ut.
	 */
	public void setDrawScores(boolean s)
	{
		m_showScore = s;
	}

	/**
	 * Ritar ut pongen
	 */
	public void drawScores(Graphics2D g, int centerx, int centery)
	{		
		g.setColor( Color.yellow );
		final int tableWidth = 200,
			   col2width = 30,
				col3width = 30;
	
		final int rowheight = 20;
		int tableheight = 20*getNumberOfRobots();

		int tableTop = (centery*2-tableheight)/2,
			tableLeft = (centerx*2-tableWidth)/2;

		class RobotInfo implements Comparable
		{
			RobotInfo(Robot r)
			{
				name = r.getName();
				frags = r.getFrags();
				kills = r.getKills();
				color = r.getUserColor();
			}

			public int compareTo(Object o) 
			{
				RobotInfo r1 = (RobotInfo) this;
				RobotInfo r2 = (RobotInfo) o;

				if( r1.frags > r2.frags ) 
					return -1;

				if( r1.frags < r2.frags ) 
					return 1;

				return (r1.kills < r2.kills) ? -1 : 1;
			}

			String name;
			int frags;
			int kills;
			Color color;
		}
		
		ArrayList info = new ArrayList();

		for(int i=0; i<getNumberOfRobots(); i++)
		{
			RobotInfo inf = new RobotInfo( getRobot(i) );

			if(getRobot(i) == getUserRobot())
				inf.name += " *";
			
			info.add( inf );
		}

		Collections.sort( info );

		for(int i=0; i<info.size(); i++)
		{
			
			RobotInfo r = (RobotInfo) info.get(i);
			g.setColor(r.color);
			g.drawString( r.name,	tableLeft, tableTop + i*rowheight );
			g.drawString( ""+r.kills,	tableLeft+(tableWidth-col2width), tableTop + i*rowheight );
			g.drawString( ""+r.frags,	tableLeft+(tableWidth-col2width-col3width), tableTop + i*rowheight );
			
		}		
	}

	/**
	 * Ritar ut "oddlighetsringat". Det vill sga de bl ringar som visar
	 * att man r oddlig.
	 */
	protected void drawImmortalRing(Graphics2D g, int centerx, int centery)
	{
		int radius=5; //centerx/5;
                
		g.setColor( Color.blue );

		//for(int i=0; i<10; i++)
		while(radius < centerx || radius < centery)
		{
			g.drawOval( centerx-radius, centery-radius, radius*2, radius*2);
			radius *= 2;
		}
	}

	/**
	 * Ritar ut en ring som visar att man har blivit trffad.
	 */
	protected void drawHit(Graphics2D g, int centerx, int centery, int tick)
	{
		int radius=centerx*tick/GameEngine.TPS*10/3+1;
                
		g.setColor( Color.red );

		g.drawOval( centerx-radius, centery-radius, radius*2, radius*2);
		radius *= 2;
	}

	/**
	 * Ritar ut radarn
	 */
	protected void drawRadar(Graphics2D g, int centerx, int centery)
	{
		int radarRadius=centerx/3;

        double radarZoom = radarRadius/(2*Robot.RADAR_VIEW_DISTANCE);
                
		g.setColor( Color.green );
		g.drawOval( centerx-radarRadius, centery-radarRadius, radarRadius*2, radarRadius*2);

		g.setColor( Color.yellow );
		drawRadarObjects(g, centerx, centery, m_robots, 3);
		
		//g.setColor( Color.blue );
		g.setColor( Color.green );
		drawRadarObjects(g, centerx, centery, m_boxes, 0);

		final Color shootcolor = new Color(255,96,96);
		g.setColor( shootcolor);
		drawRadarObjects(g, centerx, centery, m_ammos, 0);
	}

	/**
	* Ritar ut objekt p radarn
	*/
	protected void drawRadarObjects(Graphics2D g, int centerx, int centery, ArrayList objects, int radius )
	{
		int radarRadius=centerx/3;
		double radarZoom = radarRadius/(2*Robot.RADAR_VIEW_DISTANCE);
		double angle = -getUserRobot().getAngleX();
			
		for(int i = 0; i<objects.size(); i++)
		{
			Figure f = (Figure) objects.get(i); //getRobot(i);

			if( !f.isVisible() )
				continue;

			int dx = (int) -((m_robot.getCenter().getX() - f.getCenter().getX())*radarZoom);
			int dy = (int) -((m_robot.getCenter().getZ() - f.getCenter().getZ())*radarZoom);

			if( (dx*dx+dy*dy) > radarRadius*radarRadius)
				continue;

			int radarx = (int) ( dx*Math.cos(angle) - dy*Math.sin(angle) ),
				radary = (int) ( dx*Math.sin(angle) + dy*Math.cos(angle) );

			g.drawRect( centerx+radarx-radius/2, centery+radary-radius/2, radius, radius );
		}
	}
	
	/**
	 * Skapar spelrummet
	 */
	protected void createRoom()
	{
		createWalls(GameEngine.SQUAREX, GameEngine.SQUAREZ);
		createFloorOrCeiling(GameEngine.SQUAREX, GameEngine.SQUAREZ, true);
		createFloorOrCeiling(GameEngine.SQUAREX, GameEngine.SQUAREZ, false);
		addSuns();
	}

	/**
	 * Lgger till de solar som lyser upp spelrummet.
	 */
	protected void addSuns()
	{
		/* Lgg till 15 solar i en ring.
		 * int count = 15;

		for(int i=0; i<count; i++)
		{
			m_3dEngine.add( new Sun( World.minx + Math.random()*(World.maxx-World.minx),
				-4 - Math.random()*10,
				World.minz + Math.random()*(World.maxz-World.minz),
				Color.white
				)
				);
		}*/

		double  x = (GameEngine.XMAX-GameEngine.XMIN)/2,
				z = (GameEngine.ZMAX-GameEngine.ZMIN)/2;

		double dist = Math.sqrt( x*x + z*z )/2; 

		double y = GameEngine.YMAX;
		double innerDiff = (GameEngine.YMAX-GameEngine.YMIN)*.25;

		
		m_3dEngine.add( new Sun( GameEngine.XMIN+innerDiff, y-innerDiff, GameEngine.ZMIN+innerDiff, Color.white ) ); 
		m_3dEngine.add( new Sun( GameEngine.XMAX-innerDiff, y-innerDiff, GameEngine.ZMIN+innerDiff, Color.white ) ); 
		m_3dEngine.add( new Sun( GameEngine.XMIN+innerDiff, y-innerDiff, GameEngine.ZMAX-innerDiff, Color.white ) ); 
		m_3dEngine.add( new Sun( GameEngine.XMAX-innerDiff, y-innerDiff, GameEngine.ZMAX-innerDiff, Color.white ) ); 

		m_3dEngine.add( new Sun( 0, y-innerDiff, 0, Color.white ) ); 


	}

	/**
	 * Skapar en sida p en viss y-kordinat utifrn en bildfile
	 * @param ypos		Y-kordinat
	 * @param filename	Filnamn p filen
	 */
	protected void createSide(double ypos, String filename, ArrayList sidearray)
	{
        Color	col1= new Color(232, 111, 40), //(148,155,62),
			col2= new Color(128,55,22),
			base1=new Color(4,4,4), 
			base2=new Color(2, 2, 13);

		double 	xmin= GameEngine.XMIN, 
			ymin= GameEngine.YMAX,
			xmax= GameEngine.XMAX,
			ymax= GameEngine.YMIN,
			zmin= GameEngine.ZMIN,
			zmax= GameEngine.ZMAX;

		int x=20, y=30;

		PixelGrabber pg = null;

		ImageIcon icon = null;
		
		//URL url = this.getClass().getResource(filename);
		URL url = Fauke.class.getResource(filename);
		if(url != null)
		  icon = new ImageIcon(url);		
		else
		  icon = new ImageIcon(filename);
		Image image = icon.getImage();
		x=image.getWidth(null);
		y=image.getHeight(null);
		
		if(x<=0 || y<=0)
		    return;
		
		int[] pixels = new int[x*y]; //[width * height];

		pg = new PixelGrabber(image, 0, 0, x, y, pixels, 0, x);

		try 
		{
			pg.grabPixels();
		}
		catch (InterruptedException e) 
		{
			System.err.println("interrupted waiting for pixels!");
			return;
		}

				
		double xside=(xmax-xmin)/x, zside=(zmax-zmin)/y, yside=zside;

		for(int a=0; a<x; a++)
			for(int b=0; b<y; b++)
			{
				Cord3d ul = new Cord3d( -xside/2, 0, -zside/2 );
				Cord3d ur = new Cord3d( -xside/2, 0, zside/2 );

				Cord3d dr = new Cord3d( xside/2, 0, zside/2 );
				Cord3d dl = new Cord3d( xside/2, 0, -zside/2 );

				Color c = col1;
				if( (a+b)%2 == 1)
					c = col2;

				int pixel = pixels[a+b*x];
		        
				int alpha = (pixel >> 24) & 0xff;
				int red   = (pixel >> 16) & 0xff;
				int green = (pixel >>  8) & 0xff;
				int blue  = (pixel      ) & 0xff;
				c = new Color(red, green, blue);
				//System.out.println(c+" "+(a+b*x));
						
				/*if(a == x/2 && b == y/2 && i==0)
					c = Color.red;*/

				Side s = new Side(ul, ur, dr, dl, c, null);
						
				/*if(i==0)
					s.getCenter().set( xmin + (a+0.5)*xside, ymax, zmin + (b+0.5)*zside );
				else*/
					s.getCenter().set( xmin + (a+0.5)*xside, ypos, zmin + (b+0.5)*zside );

				s.setBaseColor(base1);
				if( (a+b)%2 == 1)
					s.setBaseColor(base2);

				m_3dEngine.add( s );
				sidearray.add( s );
			}
	}
	
	public void nextFloorMode()
	{
	    m_floorMode = (m_floorMode+1)%3; //(m_floorMode+2)%2 - 1;
	    createFloorOrCeiling(GameEngine.SQUAREX, GameEngine.SQUAREZ, true);
	}
	
	public void nextCeilingMode()
	{
	    m_ceilingMode = (m_ceilingMode+1)%3; //(m_floorMode+2)%2 - 1;
	    createFloorOrCeiling(GameEngine.SQUAREX, GameEngine.SQUAREZ, false);
	}

	protected void nextWallMode()
	{
	    m_wallMode = (m_wallMode+2)%3 - 1;
	    createWalls(GameEngine.SQUAREX, GameEngine.SQUAREZ);
	}
	
	protected void createFloorOrCeiling(int x, int y, boolean floor)
	{
	    ArrayList sidearray = m_floors;
	    if(!floor)
	        sidearray = m_ceilings;
	    
	    {
	        for(int i=0; i<sidearray.size(); i++)
		    {
		        m_3dEngine.remove( sidearray.get(i) );
		    }
	        sidearray.clear();
	    }
	    
	    
		double d = 20;
		double 	xmin= GameEngine.XMIN, 
			ymin= GameEngine.YMAX,
			xmax= GameEngine.XMAX,
			ymax= GameEngine.YMIN,
			zmin= GameEngine.ZMIN,
			zmax= GameEngine.ZMAX;
	
		double xside=(xmax-xmin)/x, zside=(zmax-zmin)/y, yside=zside;

	    if(floor)
	    {
	        if( m_floorMode == 2)
	            return;
	        
	        if( m_floorMode == 1)
	        {
	            createSide(ymax, "floor.gif", sidearray);
	            return;
	        }
	    }
	    else
	    {
	        if( m_ceilingMode == 2)
	            return;
	        
	        if( m_ceilingMode == 1)
	        {
	            createSide(ymin, "ceiling.gif", sidearray);
	            return;
	        }
	    }
	    
	    Color	col1= new Color(232, 111, 40),
				col2= new Color(128,55,22),
				base1=new Color(4,4,4), 
				base2=new Color(2, 2, 13);

		if(!floor)
		{
			col1=	new Color(54,65, 43);
			col2=	new Color(14,18,65);
			base1=	new Color(4,4,4);
			base2 = new Color(2, 2, 13);
		}

		for(int a=0; a<x; a++)
			for(int b=0; b<y; b++)
			{
				Cord3d ul = new Cord3d( -xside/2, 0, -zside/2 );
				Cord3d ur = new Cord3d( -xside/2, 0, zside/2 );

				Cord3d dr = new Cord3d( xside/2, 0, zside/2 );
				Cord3d dl = new Cord3d( xside/2, 0, -zside/2 );

				Color c = col1;
				if( (a+b)%2 == 1)
					c = col2;
				
				if(a == x/2 && b == y/2 && floor)
					c = Color.red;

				Side s = new Side(ul, ur, dr, dl, c, null);
				
				if(floor)
					s.getCenter().set( xmin + (a+0.5)*xside, ymax, zmin + (b+0.5)*zside );
				else
					s.getCenter().set( xmin + (a+0.5)*xside, ymin, zmin + (b+0.5)*zside );

				s.setBaseColor(base1);
				if( (a+b)%2 == 1)
					s.setBaseColor(base2);

				m_3dEngine.add( s );
				sidearray.add( s );
			}


	}

    /**
	 * Skapar vggar fr rummet.
	 */
	protected void createWalls(int x, int y)
	{
		//Hela denna funktion r full av fulkod.
	    for(int i=0; i<m_walls.size(); i++)
	    {
	        m_3dEngine.remove( m_walls.get(i) );
	    }
	    m_walls.clear();

	    if(m_wallMode <= -1)
	        return;

		double d = 20;
		double 	xmin= GameEngine.XMIN, 
			ymin= GameEngine.YMAX,
			xmax= GameEngine.XMAX,
			ymax= GameEngine.YMIN,
			zmin= GameEngine.ZMIN,
			zmax= GameEngine.ZMAX;
	
		double xside=(xmax-xmin)/x, zside=(zmax-zmin)/y, yside=zside;

		
		//Vggar p tv motsatta sidor
		{
		    yside = GameEngine.WALL_HEIGHT;
		    
			for(int i=0; i<2; i++)
			{

				//Color col1=new Color(0,128,0), col2 = new Color(0,255,0),
				//	base1=new Color(4,4,4), base2 = new Color(2, 2, 13);
			    Color col1=new Color(0,58,0), col2 = new Color(0,105,0),
				base1=new Color(4,4,4), base2 = new Color(2, 2, 13);

				if(i == 1)
				{
					//col1=new Color(255,0,0);
					//col2=new Color(128,0,0);
				    col1=new Color(105,0,0);
					col2=new Color(58,0,0);
				}

				for(int a=0; a<x; a++)
				{		
					Cord3d ul = new Cord3d( -xside/2, -yside/2, 0 );
					Cord3d ur = new Cord3d( -xside/2, yside/2, 0 );

					Cord3d dr = new Cord3d( xside/2, yside/2, 0 );
					Cord3d dl = new Cord3d( xside/2, -yside/2, 0 );

					Color c = col1;
					if( a%2 == 1)
						c = col2;

					Side s = new Side(ul, ur, dr, dl, c, null);
					s.getCenter().set( xmin + (a+0.5)*xside, ymax+yside/2, i==0 ? zmin:zmax );

					s.setBaseColor(base1);
					if( a%2 == 1)
						s.setBaseColor(base2);

					m_3dEngine.add( s );
					m_walls.add(s);
				}
			}

		}
		
		if(m_wallMode <= 0)
	        return;

		//De andra tv vggarna
		
		{
			yside = GameEngine.WALL_HEIGHT;

			for(int i=0; i<2; i++)
			{

				//Color col1=new Color(0,0,255), col2 = new Color(0,0,128),
				//	base1=new Color(4,4,4), base2 = new Color(2, 2, 13);
			    Color col1=new Color(0,0,105), col2 = new Color(0,0,58),
				base1=new Color(4,4,4), base2 = new Color(2, 2, 13);

				if(i == 1)
				{
					//col1=new Color(128,128,0);
					//col2=new Color(192,192,16);
				    col1=new Color(128/2,128/2,0);
					col2=new Color(192/4,192/4,16);
				}

				for(int a=0; a<y; a++)
				{		
					Cord3d ul = new Cord3d( 0, -yside/2, -zside/2 );
					Cord3d ur = new Cord3d( 0, yside/2, -zside/2 );

					Cord3d dr = new Cord3d( 0, yside/2, zside/2 );
					Cord3d dl = new Cord3d( 0, -yside/2, zside/2 );

					Color c = col1;
					if( a%2 == 1)
						c = col2;

					Side s = new Side(ul, ur, dr, dl, c, null);
					s.getCenter().set( i==0 ? xmin : xmax, ymax+yside/2, zmin + (a+0.5)*zside );

					s.setBaseColor(base1);
					if( a%2 == 1)
						s.setBaseColor( base2 );

					m_3dEngine.add( s );
					m_walls.add(s);

				}
			}

		}

	}

	/**
	 * Lgger till ett jobb som har skickats frn servern
	 */
	synchronized public void addJobFromServer( Job j )
	{
		if(m_netMode == CLIENT)
			m_jobs.add(j);
	}

	/**
	 * Lgger till ett jobb i jobbkn
	 */
	synchronized public void addJob( Job j )
	{
		if(m_netMode != CLIENT)
			m_jobs.add(j);
		else
		{
			//Om man r klient, skicka jobbet till servern istllet.
			m_client.send(j);
			m_client.flush();
		}

			
	}

	/**
	 * Utfr alla jobb
	 */
	synchronized protected void applyJobs()
	{
		for(int i=0; i<m_jobs.size(); i++)
			applyJob( (Job) m_jobs.get(i) );

		m_jobs.clear();
	}

	/**
	 * Utfr ett jobb
	 */
	protected void applyJob(Job j)
	{
		if(j instanceof JobJump)
			getRobot( ( (JobJump)j).m_robotId ).jump();
		else if(j instanceof JobShoot)
			getRobot( ( (JobShoot)j).m_robotId ).shoot();
		else if(j instanceof JobRecarniate)
		{
			JobRecarniate jr = (JobRecarniate) j;
			
			getRobot( jr.m_robotId ).recarniate(jr.x, jr.y, jr.z);
		}
		else if(j instanceof JobMove)
		{
			JobMove jm = (JobMove) j;
			Robot r = getRobot(jm.m_robotId);
			
			if(	isFree( r.getCenter().getX() + jm.dx,
						r.getCenter().getY() + jm.dy,
						r.getCenter().getZ() + jm.dz,
						r
					 	)
				)
			{
				r.move(jm.dx, jm.dy, jm.dz);
			}
		}
		else if(j instanceof JobRotate)
		{
			JobRotate jr = (JobRotate) j;
			
			getRobot( jr.m_robotId ).relativeRotate(jr.vx, jr.vy, jr.vz);
		}
		else if(j instanceof JobAddBox)
		{
			JobAddBox jab = (JobAddBox) j;
			
			Box box = new Box(jab.x, jab.y, jab.z, jab.type, jab.isRandom, jab.lifetime);
		
			m_boxes.add(box);
			m_3dEngine.add(box);
		}
		else if(j instanceof JobAddRobot)
		{
			JobAddRobot jar = (JobAddRobot) j;

			Robot r = new Robot(this, getNumberOfRobots(), jar.name, jar.color, jar.type);
			r.setAi( jar.ai && m_netMode != CLIENT );

			m_robots.add(r);
			m_3dEngine.add(r);
		}
	}

	/**
	 * Stter vilket ntverkslge man r i (LOCAL, SERVER eller CLIENT)
	 */
	public void setNetMode(int mode)
	{
		m_netMode=mode;
	}

	/**
	 * Stter vilket serverobjekt som ska anvndas
	 */
	public void setServer(Server s)
	{
		m_server=s;
	}

	/**
	 * Stter vilket klientobjekt som ska anvndas.
	 */
	public void setClient(Client c)
	{
		m_client=c;
	}

	/**
	 * Array med alla robotar
	 */
	protected ArrayList m_robots = new ArrayList();
	
	/**
	 * Array med alla skott
	 */
	protected ArrayList m_ammos = new ArrayList();
	
	/**
	 * Array med alla flygande sidor
	 */
	protected ArrayList m_flyingSides = new ArrayList();
	
	/**
	 * Array med alla lyssnare
	 */
	protected ArrayList m_listeners = new ArrayList();
	
	/**
	 * Array med alla ldor
	 */
	protected ArrayList m_boxes = new ArrayList();
	
	/**
	 * Array med alla jobb
	 */
	protected ArrayList m_jobs = new ArrayList();
	
	/**
	 * Array fr alla vggar
	 */
	protected ArrayList m_walls = new ArrayList();
	
	/**
	 * Array fr golv
	 */
	protected ArrayList m_floors = new ArrayList();
	
	/**
	 * Array fr tak
	 */
	protected ArrayList m_ceilings = new ArrayList();
	
	/**
	 * Vilket golvlge som anvnds
	 */
	protected int m_floorMode=0;
	
	/**
	 * Vilket taklge som anvnds
	 */
	protected int m_ceilingMode=0;
	
	/**
	 * Vilket vgglge som anvnds
	 * 
	 */
	protected int m_wallMode=1;
	
	/**
	 * Timer som styr spelet
	 */	
	protected Timer m_timer=null;
	
	/**
	 * Om pong ska ritas upp eller inte
	 */
	protected boolean m_showScore=false;
	
	/**
	 * Ntverkslge
	 */
	protected int m_netMode=LOCAL;
	
	/**
	 * Server som kontrollerar spelet
	 */
	protected Server m_server=null;
	
	/**
	 * Klient ansluten till servern
	 */
	protected Client m_client=null;
	
	/**
	 * 3d-motorn i spelet
	 */
	protected CubeEngine m_3dEngine = new CubeEngine();
	
	/**
	 * Anvndarens robot
	 */
	protected Robot m_robot=null;	

	/**
	 * ID fr olika ntverkslgen
	 */	
	static final int LOCAL=0, CLIENT=1, SERVER=2;
	
	/**
	 * Antalet tick per sekund
	 */
	public static final int    TPS = 100;
	
	/**
	 * Extrempunkter i x-led fr spelplanen
	 */
	public static final double XMIN = -10*3, XMAX=10*3;
	
	/**
	 * Extrempunkter i y-led fr spelplanen
	 */
	public static final double YMIN = -3.2, YMAX=10;
	
	/**
	 * Extrempunkter i z-led fr spelplanen
	 */
	public static final double ZMIN = -10*2, ZMAX=10*2;
	
	/**
	 * Hjden p vggen
	 */
	public static final double WALL_HEIGHT=YMAX-YMIN;
	
	/**
	 * Antalet rutor i x-led
	 */
	public static final int SQUAREX = 13*3/2;
	
	/**
	 * Antalet rutor i z-led
	 */
	public static final int SQUAREZ = 13;
	
	/**
	 * Graviation
	 */
	public static final double GRAVITY = -0.9/GameEngine.TPS;
    
}
