import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;


/**
 * GameLauncher r sjlva startfnstret, dr man gr diverse
 * instllningar och startar spelet
 * @author Per-Erik Kristensson
 */
public class GameLauncher extends JFrame implements ActionListener
{

    /**
     * Skapar en GameLauncher
     */
    public GameLauncher()
	{
		super("Fauke");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		//Vlj en slumpmssig startfrg
		Color startColor = new Color(	(int)(128 + 128*Math.random()),
										(int)(128 + 128*Math.random()),
										(int)(128 + 128*Math.random())
									  );
		
		m_color.setBackground(startColor);
		m_robotPick = new RobotPickPanel(startColor);
		
		JPanel superpane = new JPanel();
		superpane.setLayout( new GridLayout(1, 2) );
		
		JPanel rightpane = new JPanel();
		rightpane.setLayout( new BorderLayout() );
		rightpane.add( m_robotPick, BorderLayout.CENTER );
		
		JPanel leftpane = new JPanel();
		leftpane.setLayout( new GridLayout(8,1) );

		leftpane.add(m_name);

				
		m_color.addActionListener(this);
		leftpane.add(m_color);

		leftpane.add( new JLabel("Vlj speltyp: ") );

		{
			ButtonGroup radios = new ButtonGroup();

			m_radio_server.setSelected(true);

			radios.add( m_radio_local );
			radios.add( m_radio_server );
			radios.add( m_radio_client );

			leftpane.add( m_radio_local );
			leftpane.add( m_radio_server );
			leftpane.add( m_radio_client );
		}

		{
			JPanel p2 = new JPanel();
			p2.setLayout( new GridLayout(1, 2));
			p2.add( new JLabel("Serveradress:"));
			p2.add( m_server );
			
			leftpane.add( p2 );
		}
		
		JButton start = new JButton("Starta");
		start.addActionListener(this);
		leftpane.add( start );	
		
		
		superpane.add(leftpane);
		superpane.add(rightpane);
		//setContentPane(superpane);
		
		JPanel duperpane = new JPanel();
		duperpane.setLayout( new BorderLayout() ); //new GridLayout(2, 1) );
		m_introPanel = new IntroPanel();
		
		duperpane.add(m_introPanel, BorderLayout.CENTER);
		duperpane.add(superpane, BorderLayout.SOUTH);
		
			
		setContentPane(duperpane);
		
		pack();

		setVisible(true);
	}

	public void actionPerformed(ActionEvent e)
	{
	    //Byta frg?
		if(e.getSource() == m_color)
		{
			m_color.setBackground( JColorChooser.showDialog(this, "Vlj en frg", m_color.getBackground() ) );
			m_robotPick.setUserColor(m_color.getBackground());
			return;
		}

		//Annars, startat spelet p ngot stt
		m_robotPick.stop();
		m_introPanel.stop();
		
		if(m_radio_local.isSelected())
		{
		    //Lokalt spel
			GameEngine engine = new GameEngine();
			engine.addRobotDirectly( new Robot(engine, 0, m_name.getText(),  m_color.getBackground(), m_robotPick.getType()) );
			GameFrame game = new GameFrame(engine);
		}
		else if(m_radio_server.isSelected())
		{
		    //Server
		    GameEngine m_engine = new GameEngine();		    
			m_engine.setNetMode(GameEngine.SERVER);
			
			Server m_server = new Server();
			m_server.setGameEngine(m_engine);
			m_server.start();
			
			m_engine.addRobotDirectly( new Robot(m_engine, 0, m_name.getText(), m_color.getBackground(), m_robotPick.getType() ) );
		    
			m_engine.setServer(m_server);

			GameFrame game = new GameFrame(m_engine);

		}
		else if(m_radio_client.isSelected())
		{
			GameConnector g = new GameConnector(m_name.getText(), m_color.getBackground(), m_robotPick.getType(), m_server.getText());
		}

		setVisible(false);

	}

	/**
	 * Knappalternativ
	 */
	protected JRadioButton m_radio_local = new JRadioButton("Lokalt spel"),
	 m_radio_server = new JRadioButton("Server"),
	 m_radio_client = new JRadioButton("Klient");

	/**
	 * Textflt fr anvndarnamn och namn p servern
	 */
	protected JTextField	 m_name = new JTextField("Robert"),
		 					m_server = new JTextField("localhost");
	
	/**
	 * Knapp dr man stller in frgen.
	 */
	protected JButton m_color = new JButton("Frg");
	
	/**
	 * Panel dr man vljer robottyp.
	 */
	protected RobotPickPanel m_robotPick;
	
	protected IntroPanel m_introPanel;

}