import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.JPanel;

import cubeengine.Camera;

/**
 * GamePanel r en panel fr Fauke. Den ritar upp skrmen och tar
 * han om anvndarens styrningar.
 * '
 * @author Per-Erik Kristensson
 */
public class GamePanel extends JPanel 
implements MouseMotionListener, MouseListener, KeyListener, GameEngineListener
{

	/**
	 * Skapar en spelpanel
	 */
	public GamePanel(GameEngine engine)
	{
		super();
		addMouseMotionListener(this);
		addMouseListener(this);
		addKeyListener(this);

		m_gameEngine = engine;
		m_gameEngine.addGameEngineListener(this);
	}
	
	/**
	 * Anropas nr spelmotorn har uppdaterats. D ska roboten styras
	 * och skrmen ritas om.
	 */
	public void engineUpdated()
	{
		if(m_moveForward) 	m_gameEngine.getUserRobot().tryMoveForward();
		if(m_moveBackward)  m_gameEngine.getUserRobot().tryMoveBackward();
		if(m_moveLeft)		m_gameEngine.getUserRobot().tryMoveLeft();
		if(m_moveRight)		m_gameEngine.getUserRobot().tryMoveRight();
		if(m_moveUp)		m_gameEngine.getUserRobot().tryMoveUp();
		if(m_moveDown)		m_gameEngine.getUserRobot().tryMoveDown();
		
		final double turnAngle = Math.PI*2*1/360;

		if(m_turnUp)		m_gameEngine.getUserRobot().tryRotate(0, -turnAngle, 0);	
		if(m_turnDown)		m_gameEngine.getUserRobot().tryRotate(0, turnAngle, 0);	
		if(m_turnLeft)		m_gameEngine.getUserRobot().tryRotate(turnAngle, 0, 0);	
		if(m_turnRight)		m_gameEngine.getUserRobot().tryRotate(-turnAngle, 0, 0);	

		if(m_shoot)			shootOrRecarniate();
		
		m_tpsCounter++;
		
		if(m_tpsCounter >= GamePanel.TPS_BEFORE_REDRAW)
		{
			repaint();
			m_tpsCounter = 0;
		}
	}

	/**
	 * Mste finnas annars tappar panelen fokus.
	 */
	public boolean isFocusCycleRoot()
	{
		return true;
	}

	/**
	 * Mste finnas annars tappar panelen fokus.
	 */
	public boolean isFocusTraversable() 
	{
		return true;
	}

	/**
	 * Nr en tangent har trycks ner, kontrollera vad som ska gras.
	 */
	public void keyPressed(KeyEvent e) 
	{
		setKeyBindValue(e, true);
	}

	/**
	 * Nr en tangent har slppts, kontrollera vad som ska gras.
	 */
	public void keyReleased(KeyEvent e) 
	{
		setKeyBindValue(e, false);
	}

	/**
	 * Anropas om en tangents har slppts eller trycks ned
	 * (avgrs av b):
	 */
	public void setKeyBindValue(KeyEvent e, boolean b)
	{
		if(e.getKeyCode() == KeyEvent.VK_W) m_moveForward = b;
		if(e.getKeyCode() == KeyEvent.VK_S) m_moveBackward = b;
		if(e.getKeyCode() == KeyEvent.VK_A) m_moveLeft= b;
		if(e.getKeyCode() == KeyEvent.VK_D) m_moveRight= b;
		if(e.getKeyCode() == KeyEvent.VK_R) m_moveUp= b;
		if(e.getKeyCode() == KeyEvent.VK_F) m_moveDown= b;

		if(e.getKeyCode() == KeyEvent.VK_H) m_turnUp = b;
		if(e.getKeyCode() == KeyEvent.VK_N) m_turnDown = b;
		if(e.getKeyCode() == KeyEvent.VK_J) m_turnLeft= b;
		if(e.getKeyCode() == KeyEvent.VK_K) m_turnRight= b;

		if(e.getKeyCode() == KeyEvent.VK_SPACE)
		{
			m_shoot = b;
		}

		if(e.getKeyCode() == KeyEvent.VK_SHIFT) m_gameEngine.setDrawScores(b);		
	}

	/**
	 * Nr man skriver ngot.
	 */
	public void keyTyped(KeyEvent e) 
	{
		if(e.getKeyChar() == 'o')  m_gameEngine.getUserRobot().tryGetOverview();

		if(e.getKeyChar() == 'c' )  m_gameEngine.getUserRobot().tryCenterView();
		if(e.getKeyChar() == 'x' )  m_gameEngine.getUserRobot().tryJump();
		
		if(e.getKeyChar() >= '6' && e.getKeyChar() <= '7')
		{
			Camera camera = m_gameEngine.getUserRobot();
			
			if(e.getKeyChar() == '6') camera.setZoom( camera.getZoom() - 1);
			if(e.getKeyChar() == '7') camera.setZoom( camera.getZoom() + 1);			
		}
		
		if(e.getKeyChar() == 'y' ) m_gameEngine.m_robot = m_gameEngine.getNextRobot( m_gameEngine.m_robot );
		
		if(e.getKeyChar() == '+' ) m_gameEngine.addRobot(true);
		
		if(e.getKeyChar() == '8' ) m_gameEngine.nextWallMode();
		if(e.getKeyChar() == '9' ) m_gameEngine.nextFloorMode();
		if(e.getKeyChar() == '0' ) m_gameEngine.nextCeilingMode();
	}

	/**
	 * Begr att roboten ska skjuta om den lever, annars
	 * testas terupplivning
	 */
	public void shootOrRecarniate()
	{
		Robot r = m_gameEngine.getUserRobot();
		
		if(r.canRecarniate())
		{
			r.tryRecarniate();
			m_shoot=false;
		}
		else
			r.tryShoot();		
	}


	/**
	 * Ritar ut skrmen
	 */
	public void paint(Graphics g1)
	{
		if(m_gameEngine.getUserRobot() == null)
		{
			if(m_gameEngine.getNumberOfRobots() > 0)
			{
				m_gameEngine.setUserRobot( m_gameEngine.getRobot(0) );
			}
			else
				return ;
		}

		Graphics2D g = (Graphics2D) g1;


		Rectangle bound = getBounds();

		if(!bound.equals(m_lastBound))
		{
			//Storleken har frndrats, ska ny bildbuffer.
			m_myBuffer = createImage(bound.width, bound.height);
			m_lastBound = bound;
		}

		m_myBuffer.getGraphics().setColor( Color.white );
		m_myBuffer.getGraphics().fillRect( 0, 0, bound.width, bound.height );

		//Rita upp spelet...
		m_gameEngine.render( (Graphics2D) m_myBuffer.getGraphics(), bound.width/2, bound.height/2 );

		//...och kopiera det till skrmen.
		g.drawImage(m_myBuffer, 0, 0, this);
		
		//Skriv ut FPS
		m_fpsCounter++;;
		long endTime = System.currentTimeMillis();
		if(m_startTime == 0)
			m_startTime=endTime;
		
		double fps = (double)m_fpsCounter/( endTime-m_startTime) * 1000.0;
		
		g.setColor(Color.white);
		g.drawString( "FPS: " + Math.round(100*fps)/100.0, 10, bound.height-12);
		
		if(m_fpsCounter > 25)
		{
			m_fpsCounter=0;
			m_startTime=endTime;
		}
	}

	public void mouseDragged(MouseEvent e){}

	public void mouseMoved(MouseEvent e) 
	{
		Rectangle bound = getBounds();

		if(m_mouseMove)
		{
			double diffx = e.getX()-m_lastMouseX, diffy= e.getY()-m_lastMouseY;

			m_gameEngine.getUserRobot().tryRotate( -diffx/100/60*6*2, diffy/100*2/60*6*2, 0);
		}

		m_lastMouseX=e.getX();
		m_lastMouseY=e.getY();
	}

	public void mouseClicked(MouseEvent e) 
	{ 
		grabFocus();

		//Vnsterklick
		if(e.getButton() == MouseEvent.BUTTON1)
				shootOrRecarniate();
		
		//Hgerklick
		if(e.getButton() == MouseEvent.BUTTON3)
			m_mouseMove = !m_mouseMove;
	}

	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}


	/**
	 * Spelmotor som anvnds
	 */
	protected GameEngine m_gameEngine=null;
	
	/**
	 * Buffert fr skrmen
	 */
	protected Image m_myBuffer = null;
	
	/**
	 * Storlek p bufferten vid senaste uppritning
	 */
	protected Rectangle m_lastBound = null;
	

	/**
	 * Om man styr med musen
	 */
	protected boolean m_mouseMove = false;

	/**
	 * Flytta framt?
	 */
	protected boolean m_moveForward = false;
	/**
	 * Flytta bakt?
	 */
	protected boolean m_moveBackward = false;
	/**
	 * Flytta t vnster
	 */
	protected boolean m_moveLeft= false;
	/**
	 * Flytta t hger
	 */
	protected boolean m_moveRight= false;
	/**
	 * Flytta uppt?
	 */
	protected boolean m_moveUp=false;
	/**
	 * Flytta nedt?
	 */
	protected boolean m_moveDown=false;

	/**
	 * Vnd uppt?
	 */
	protected boolean m_turnUp=false;
	/**
	 * Vnd nedt?
	 */
	protected boolean m_turnDown=false;
	/**
	 * Vnd t vnster?
	 */
	protected boolean m_turnLeft=false;
	/**
	 * Vnd t hger?
	 */
	protected boolean m_turnRight=false;
	
	/**
	 * Skjuta?
	 */
	protected boolean m_shoot=false;

	/**
	 * Antalet tick sedan fregnde uppritning
	 */
	protected int m_tpsCounter=0;
	/**
	 * Muskordinaterna vid senaste ndring
	 */
	protected double m_lastMouseX=0, m_lastMouseY=0;
	
	/**
	 * Antalet uppritning
	 */
	protected int m_fpsCounter = 0;
	
	/**
	 * Starttid fr uppritning fr aktuell mtning
	 */
	protected long m_startTime = 0;
	
	/**
	 * Antal tick innan uppritning
	 */
	public static final int		TPS_BEFORE_REDRAW=2;

}
