import java.awt.Color;
import java.io.Serializable;

import cubeengine.Camera;
import cubeengine.Cord3d;
import cubeengine.Figure;
import cubeengine.Sun;

/**
 * En Robot r sjlva spelaren i spelet. Fr varje tick
 * i spelet ska tick anropas. D gr Roboten de ndvndiga
 * frflyttning och berkningar som ska gras fr varje tick. Om 
 * roboten r ai utfr den ven ett steg.
 * 
 * Fr att styra roboten finns det ett antal try-funktioner att 
 * anropa, man ska INTE anropa t ex shoot fr att skjuta, istllet
 * tryShoot. Detta fr att roboten d lgger till ett Job i spelmotorn,
 * som sedan anropar shoot nr det r dags. P s stt kan man ltt
 * hlla ntverksspelet synkroniserad.
 * 
 * 
 * @author Per-Erik Kristensson
 */
public class Robot extends Camera implements Serializable
{
    /**
     * Skapar en Robot
     * @param engine Spelmotor som Roboten kr p
     * @param id	 Robotens ID-nummer
     * @param name	 Robotens namn
     * @param color  Robotens frg
     * @param type	 Robotens type (0 eller 1)
     */
	public Robot(GameEngine engine, int id, String name, Color color, int type)
	{
		super();
                
		m_id = id;
		m_name = name;
		
		setUserColor(color);
		setRobotType(type);
	
		m_engine=engine;

		//Brja som dd, fast utan animation
		kill();
		m_deadTickCount = Robot.DEATH_ANIMATION;
		m_kills=0;
	}
	
	/**
	 * Stter vilken spelmotor som ska anvndas
	 * @param engine spelmotor som ska anvndas
	 */
	public void setEngine(GameEngine engine)
	{
	    m_engine = engine;
	}
	
	/**
	 * Stter vilken typ av robot, och laddar in
	 * rtt figur.
	 * 
	 * @param type Robottyp
	 */
	public void setRobotType(int type)
	{
		m_type=type;
		load("robot"+(type+1));
		setUserColor(m_userColor);
	}

	/**
	 * Hmtar robottyp
	 * @return Robottyp
	 */
	public int getRobotType()
	{
		return m_type;
	}

	/**
	 * Stter robotens anvndarfrg. Denna frg
	 * visas p en av robotens sidor
	 * @param color Ny anvndarfrg.
	 */
	public void setUserColor(Color color)
	{
		m_userColor = color;
		getSide(0).setColor( m_userColor  );
	}
	
	/**
	 * Kollar om roboten blir trffad av ammunition,
	 * samt rknar ned liv om den blir det. Kolla 
	 * returvrdet fr att f konsekvenserna.
	 * 
	 * @param am Ammunition att testa p
	 * @return NOHIT om ingen trff, HURTHIT om trff men inte ddlig, KILLHIT och ddlig trff.
	 */
	public int hitTest(Ammo am)
	{
		if( 	am.getShooterID() != m_id &&
				isAlive() &&
				isInside(am) 
			)
		{
			if(!isImmortal())
			{
				m_lastHitTick = 0;

				m_health -= am.getKillValue();
				
				if(m_health <=0)
				{
					kill();
					//am.getSender().addFrag();
					m_engine.getRobot( am.getShooterID() ).addFrag();
					m_health = 0;
					return KILLHIT;
				}
			}
			
			return HURTHIT;
		}
		
		return NOHIT;
	}
	
	/**
	 * Hmtar hur mnga tick kvar robotens radar kommer
	 * att fungera. 0 om ingen radar.
	 *
	 * @return radarvrde.
	 */
	public int getRadarValue()
	{
		return m_radarTickCount;
	}

	/**
	 * Kollar om roboten r oddlig.
	 * @return true om roboten r oddlig
	 */
	public boolean isImmortal()
	{
		return m_immortalTickCount > 0;
	}
	
	/**
	 * Kollar om en robot rr en lda, och plockar
	 * upp innehllet om den gr det.
	 * 
	 * @param b Lda att testa p
	 * @return true om ldan b plockas b
	 */
	public boolean boxHitTest(Box b)
	{
		if( isAlive() && isInside(b) )
		{
			if(m_target == b)
				m_target = null;

			//Plocka upp innehllet.
			if(b.getType() == Box.RADAR)
			{
				m_radarTickCount += Robot.RADAR_LIFETIME;
			}
			else if(b.getType() == Box.IMMORTAL )
			{
				m_immortalTickCount = Robot.IMMORTAL_LIFETIME;
				
				setGlowColor(Color.blue);
			}
			else if(b.getType() == Box.AMMO)
			{
				m_ammo += Box.BOX_AMMO_COUNT;

				if(m_ammo > Robot.MAX_AMMO)
					m_ammo = Robot.MAX_AMMO;

			}
			else
			{
				m_health += Box.BOX_HEALTH;
				
				if(m_health > Robot.MAX_HEALTH)
					m_health = Robot.MAX_HEALTH;
			}
				
			return true;			
		}
		
		return false;				
	}

	/**
	 * Begr att robotens ska sttas i "versiktslge",
	 * s att hela planen syns uppifrn. Detta fungerar
	 * endast om roboten r dd.
	 */
	public void tryGetOverview()
	{
		if(!isAlive())
		{
		    //Man ska kunna strunta i try-funktionerna hr,
		    //d en dd robots position inte r relevanta
		    //fr andra datorer.
			//tryCenterView();
		    final double x = GameEngine.XMAX*0.75;
		    final double y = GameEngine.YMAX*0.85;
		    final double z = GameEngine.ZMAX*0.75;
		    
			getCenter().set(x, y, z);
			tryRotate(	Math.atan2(x, z)+getAngleX(), 
			        	Math.atan2(y, Math.sqrt(x*x+z*z))+getAngleY(),
			        	0);
			
		}
	}
	

	/**
	 * Begr att roboten ska skjuta.
	 */
	public void tryShoot()
	{
		if(m_ammoDelayTickCount <= 0 && m_ammo > 0 && isAlive())
		{			
			m_engine.robotShoot(this);
		}
	}

	/**
	 * Skjuter ivg ammunition
	 */
	public void shoot()
	{
		if( ! (m_ammoDelayTickCount <= 0 && m_ammo > 0 && isAlive()) )
			return;
		
		m_ammoDelayTickCount=Robot.AMMO_DELAY;
		m_ammo--;

		double[][] matrix = Cord3d.createRotateMatrix(getAngleX(), getAngleY(), -getAngleZ()*0);	

		double dx=0, dy=0, dz=-Ammo.AMMO_SPEED;

		double vx = dx * matrix[0][0] + dy*matrix[0][1] + dz*matrix[0][2];
		double vy = dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2];
		double vz = dx * matrix[2][0] + dy*matrix[2][1] + dz*matrix[2][2];

               
		Ammo am = null;
		
		if(!isImmortal())
		    am = new Ammo( this, new Color(255,218,140) );
		else
		    am = new Ammo( this, new Color(40,168,255) );

		m_engine.addAmmo(am);

	}
	
	/**
	 * Begr att roboten ska hoppa.
	 */
	public void tryJump()
	{
		m_engine.robotJump(this);
	}

	/**
	 * Gr att roboten hoppar.
	 */
	public void jump()
	{
		if(!isAlive())
			return;

		if(m_speedy == 0)
		{
			m_speedy=JUMP_SPEED;
		}
	}
	
	
	/**
	 * Begr att roboten ska flyttas framt.
	 */
	public void tryMoveForward()
	{
		tryRelativeMove(0, 0, -MOVE_SPEED);
	}

	/**
	 * Begr att roboten ska flyttas bakt
	 */
	public void tryMoveBackward()
	{
		tryRelativeMove(0, 0, MOVE_SPEED);
	}
	
	/**
	 * Begr att roboten ska flyttas t vnster.
	 */
	public void tryMoveLeft()
	{
		tryRelativeMove(-MOVE_SPEED, 0, 0);
	}

	/**
	 * Begr att roboten ska flyttas t hger.
	 */
	public void tryMoveRight()
	{
		tryRelativeMove(MOVE_SPEED, 0, 0);
	}


	/**
	 * Begr att roboten ska flyttas t ned (fungerar endast om man r dd).
	 */
	public void tryMoveDown()
	{
		if(!isAlive() && m_deadTickCount > Robot.DEATH_ANIMATION)
			tryMove(0, -MOVE_SPEED, 0);
	}

	/**
	 * Begr att roboten ska flyttas t uppt (fungerar endast om man r dd).
	 */
	public void tryMoveUp()
	{
		if(!isAlive() && m_deadTickCount > Robot.DEATH_ANIMATION)
			tryMove(0, MOVE_SPEED, 0);
	}


	/**
	 * Begr att roboten ska flyttas utifrn den vinkel
	 * den r riktad mot.
	 * 
	 * @param dx Skillnad i x-led
	 * @param dy Skillnad i y-led
	 * @param dz Skillnad i z-led
	 */
	public void tryRelativeMove(double dx, double dy, double dz)
	{
		double[][] matrix = Cord3d.createRotateMatrix(getAngleX(), getAngleY(), -getAngleZ()*0);	

		double  addx = dx * matrix[0][0] + dy*matrix[0][1] + dz*matrix[0][2],
				//Omjligt att rra sig i hjdled
				addy = 0, //dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2],
				addz = dx * matrix[2][0] + dy*matrix[2][1] + dz*matrix[2][2];
		
		tryMove(addx, addy, addz);
	}
	
	/**
	 * Begr att roboten ska flyttas.
	 * 
	 * @param dx Skillnad i x-led 
	 * @param dy Skillnad i y-led
	 * @param dz Skillnad i z-led
	 */
	public void tryMove(double dx, double dy, double dz)
	{
		//Om man r dd, utfr rrelse direkt
		if(!isAlive())
		{
			if(m_deadTickCount > Robot.DEATH_ANIMATION)
			{
				getCenter().add(dx, dy, dz);
			}
		}
		else
		if(	m_engine.isFree( getCenter().getX() + dx,
							getCenter().getY() + dy,
							getCenter().getZ() + dz,
							this
							)
			)
		{
			m_engine.robotMove(this, dx, dy, dz);
		}
	}
	
	/**
	 * Begr att roboten ska roteras (runt sin egen axel).
	 * 
	 * @param dx Skillnad i x-led
	 * @param dy Skillnad i y-led
	 * @param dz Skillnad i z-led
	 */
	public void tryRotate(double dx, double dy, double dz)
	{
		m_engine.robotRotate(this, dx, dy, dz);
	}
	/**
	 * Flyttar roboten 
	 * @param dx Skillnad i x-led
	 * @param dy Skillnad i y-led
	 * @param dz Skillnad i z-led
	 */
	public void move(double dx, double dy, double dz)
	{
		if(!isAlive())
			return;	

		getCenter().add(dx, dy, dz);
	}	

	/**
	 * Frsker centrera blicken. Detta ger inte alltid
	 * ett lyckat resultat, man fr ibland frska flera gnger.
	 */
	public void tryCenterView()
	{
		tryRotate( 0, getAngleY(), 0 );
	}
	
	/**
	 * Begr att teruppliva roboten
	 */
	public void tryRecarniate()
	{
		if(canRecarniate())
			m_engine.robotRecarniate(this); 		
	}

	/**
	 * Kollar om roboten kan terupplivas
	 * @return true om roboten kan terupplivas
	 */
	public boolean canRecarniate()
	{
		if(!isAlive() && m_deadTickCount >= ticksBeforeRecarniation)
			return true;
		
		return false;
	}

	/**
	 * terupplivar roboten p plats (x,y,z), och vrider
	 * roboten mot (x,y,z).
	 * 
	 * @param x Position x
	 * @param y Position y
	 * @param z Position z.
	 */
	public void recarniate(double x, double y, double z)
	{
		setGlowColor(null);

		setVisible(true);
        m_target=null;
                
        m_radarTickCount = 0;
        m_ammoDelayTickCount = 0;
        m_immortalTickCount = 0;
        m_health = Robot.START_HEALTH;
		m_ammo=Robot.START_AMMO;
		m_lastHitTick=GameEngine.TPS;
		getCenter().set(x, y, z);
        
		final double maxdiffangle = Math.PI*2*0.001/360;
		double aa = getAngleY();
		
		//Vrid upp roboten s att alla vinklar r 0, med en 
		//maximal skillnad p en tusendels grad.
		while( 	getAngleX() < -maxdiffangle ||
		        getAngleY() < -maxdiffangle ||
		        getAngleZ() < -maxdiffangle ||
		        getAngleX() > maxdiffangle ||
		        getAngleY() > maxdiffangle ||
		        getAngleZ() > maxdiffangle )
		{
		    relativeRotate(getAngleX(), getAngleY(), getAngleZ());
		}
		
		//Stt vinkeln y p ett hrt stt
		m_vy = 0;
		m_vx = 0;
		m_vz = 0;
		
		//relativeRotate(getAngleX(), getAngleY(), getAngleZ());

        //tryCenterView();

		//Vrid roboten mot centrum
		rotateToCenter();

		m_isAlive=true;
		m_speedy=0;
	}
	
	/**
	 * Ddar roboten.
	 */
	public void kill()
	{
		setGlowColor(null);
		m_kills++;
		setVisible(false);
		m_isAlive=false;
		m_deadTickCount=0;
		m_speedy=0;
	}
	
	/**
	 * Roterar roboten mot centrum (0, 0, 0)
	 */
	public void rotateToCenter()
	{
		relativeRotate( Math.atan2( getCenter().getX(), getCenter().getZ()), 0, 0);
	}

	/**
	 * Kollar hur lnge sedan det var roboten blev trffad
	 * @return hur lnge sedan roboten blev trffad
	 */
	public int getLastHitTick()
	{
		return m_lastHitTick;
	}

	/**
	 * Kollar hur mycket ammonution som r kvar
	 * @return antal skott som r kvar
	 */
	public int getAmmo()
	{
		return m_ammo;
	}
	
	
	/**
	 * Kollar om ai:n tycker att ammunition r farligt nra.
	 * @return true om ngon farlig ammunition r nra.
	 */
	protected boolean thinkIsDangerAmmoClose()
	{
		final double dist = 3.25;
		
		for(int i=0; i<m_engine.m_ammos.size(); i++)
		{
			Ammo ammo = (Ammo) m_engine.m_ammos.get(i);
			
			//if( ammo.getSender() != this && getCenter().getDistance(ammo) < dist )
			if( ammo.getShooterID() != m_id && getCenter().getDistance(ammo) < dist )
			{
				//Sikta mot denna?
				/*if(ammo.getSender().isAlive())
					m_target = ammo.getSender();*/
			    if( m_engine.getRobot( ammo.getShooterID() ).isAlive())
					m_target = m_engine.getRobot( ammo.getShooterID() );
				
				return true;
			}
		}
		
		return false;
	}

	/**
	 * Testar om ai:n tycker att den har dlig hlsa
	 * @return true om roboten har dlig hlsa
	 */
	protected boolean thinkHasBadHealth()
	{
		return (m_health < Ammo.MAX_AMMO_KILL*0.85);
	}
	
	/**
	 * Testar om ai:n tycker att den ska byta ml.
	 * Den gr s om s r fallet.
	 */
	protected void thinkPickNewTarget()
	{
		if(m_target != null && !m_target.isVisible())
			m_target = null;
		
		if(thinkHasBadHealth() || m_ammo <= 0)
		{
			int boxcount = m_engine.getNumberOfBoxes();
			int wantedType = Box.AMMO;

			if(thinkHasBadHealth())
				wantedType = Box.HEALTH50;
			
			if(boxcount > 0)
			{
				//Vlj den nrmaste ldan an den nskade typen.
				double distance = 1000000; 
				
				for(int i=0; i<boxcount; i++)
				{
					Box box = m_engine.getBox(i);
					
					if( distance > getDistance(box) &&
						( box.getType() == wantedType || box.getType() == Box.IMMORTAL || box.isRandom() )
					  )
					{
						m_target = box;
						distance = getDistance(m_target);
					}
				}	
			
				if(m_target != null)
					return;
			}
		}
		
		if( 	m_target == null || 
			( m_target instanceof Robot && (!((Robot)m_target).isAlive() ||
				((Robot)m_target).isImmortal()))
			)
		{
			int n = (int) (Math.random()*m_engine.getNumberOfRobots());
			
			if(m_engine.getRobot(n) != this && m_engine.getRobot(n).isAlive())
				m_target = m_engine.getRobot(n);
			
		}
	}
	
	/**
	 * Tnker. Gr en strategisk rrelse (nja, man kan ju ltsas).
	 */
	protected void think()
	{
		
		if(canRecarniate())
			tryRecarniate();
		
		if(thinkIsDangerAmmoClose())
			tryJump();
	
		thinkPickNewTarget();
		
		if(m_target != null )
		{
			
			double dx = getCenter().getX() - m_target.getCenter().getX();
			double dz = getCenter().getZ() - m_target.getCenter().getZ();
			double distance = Math.sqrt(dx*dx + dz*dz);
			
			double targetAngle = Math.atan2(-dz, -dx) + Math.PI/2;
			double diffAngle = -( normalizeAngle(getAngleX() - targetAngle));
						
			if( Math.abs( normalizeAngle(getAngleX() - targetAngle)) > Math.PI*5/360 )
			{
				//Rotera med hastigheten 1/2 varv per sekund som mest
				final double maxRotateSpeed = Math.PI*2*0.5/GameEngine.TPS;
				

			
				if(diffAngle > maxRotateSpeed)
					diffAngle = maxRotateSpeed;
				else if(diffAngle <-maxRotateSpeed)
					diffAngle= -maxRotateSpeed;

				
				//Rr sig i sidled om skillnad ver 15 grader
				if(Math.abs(diffAngle) > Math.PI*2*15/360)
					if(diffAngle < 0)
						tryMoveLeft();
					else
						tryMoveRight();
			
				
				tryRotate( -diffAngle , 0, 0 );
				
			}	
			
			//Sikta upp/ned?
			/*if( m_target instanceof Robot &&
				((Robot)m_target).isAlive())*/
			{
				
					
				double targetVerticalAngle = -Math.atan2( (getCenter().getY() - m_target.getCenter().getY()), distance  );
	
	                        //System.out.println(targetAngle + " ; "  + getAngleY());
	                        
				if( Math.abs(getAngleY() - targetVerticalAngle) > Math.PI*2/360 )
				{
					//Rotera med hastigheten 1 varv per sekund som mest
					final double maxRotateSpeed = Math.PI*2/GameEngine.TPS;
					double r = -(getAngleY() - targetVerticalAngle);
	                                r = getAngleY()-targetVerticalAngle;
	
				
					if(r > maxRotateSpeed)
						r = maxRotateSpeed;
					else if(r <-maxRotateSpeed)
						r= -maxRotateSpeed;
				
					
					tryRotate( 0, r, 0 );
				}	
				//else if( Math.abs( targetVerticalAngle ) 
					//tryCenterView();
			}

			if(m_target instanceof Box)
			{
				if( Math.abs(diffAngle) < Math.PI*2*20/360 )
					tryMoveForward();
			}
			else
			{
				if(distance > 10 && Math.abs(diffAngle) < Math.PI*2*20/360)
					tryMoveForward(); //(0, 0, -0.2);
				else if(distance < 9 || Math.abs(diffAngle) > Math.PI*2*50/360)
					tryMoveBackward(); //tryMove(0, 0, 0.2);
				
				/*double x = Math.abs( ( (getAngleX()-targetAngle))%(Math.PI*2)  );
				if(x>Math.PI)
					x-=Math.PI;*/
				double x = normalizeAngle( getAngleX()-targetAngle );
				final double maxShootAngle = Math.PI*2*15/360;
				
				/*while(x > Math.PI*2)
					x -= Math.PI*2;*/
				
				if(	isAlive() && ( x<maxShootAngle && x>-maxShootAngle ))
					tryShoot();
			}
			
				
		}
                
	}
	
	/**
	 * Normaliserar en vinkel s att den r mellan -Pi och +Pi
	 * @param angle - Vinkel
	 * @return Retunerar vinkeln s den ligger mellan -Pi och +Pi.
	 */
	static protected double normalizeAngle(double angle)
	{
		angle = angle%(Math.PI*2);
		if(angle > Math.PI)
			angle -= 2*Math.PI;
		if(angle < -Math.PI)
			angle += 2*Math.PI;
		
		return angle;
	}
	
	/**
	 * Stter en gldande frg p roboten. Detta innebr tv saker:
	 * 1. En osynlig sol lggs till mitt i roboten som lyser upp vrlden.
	 * 2. Alla kanter stts till den gldande frgen.
	 *  	
	 * @param glow
	 */
	public void setGlowColor(Color glow)
	{
		if(glow == m_glowColor)
			return;
		
		//Ta bort?
		if(glow == null)
		{
			m_engine.removeExtraSun(m_glowSun);
		}
		//Lgg till
		else
		{			
			m_glowSun = new Sun();
			m_glowSun.setColor(glow);
			m_glowSun.setCenter( getCenter() );
			m_glowSun.setVisible(false);
			m_glowSun.setGamma(5.0);
			m_engine.addExtraSun(m_glowSun);
		}
		
		m_glowColor = glow;
		setAllSideBorderBaseColor(m_glowColor);
	}
	
	/**
	 * Tickar roboten. Frflyttar den, rknar ned 
	 * radar och liknande.
	 */
	public void tick()
	{
		if(m_radarTickCount > 0)
		{
			m_radarTickCount--;
		}

		m_lastHitTick++;

		if(!isAlive())
		{
			m_deadTickCount++;

			if(m_deadTickCount < Robot.DEATH_ANIMATION)
			{
				double dy = getAngleY()-(-Math.PI/4);
				relativeRotate( Math.PI*2/Robot.DEATH_ANIMATION*0.3, dy*0.12, 0);
				return;
			}
		}
		
		if(m_ammoDelayTickCount > 0)
		{
			m_ammoDelayTickCount--;
		}
		

		
		if(m_immortalTickCount > 0)
		{
			m_immortalTickCount--;
			
			if(m_immortalTickCount <= 0)
				setGlowColor(null);
		}
		
		if(m_isAi)
			think();
		
		if(isAlive())
		{
			double ypos = getCenter().getY();
			
			//I luften?
			if(ypos >= 0)
			{
				if(getCenter().getY() + m_speedy > 0)
				{
				    //Kan flyga upp/ned?
				    if( (m_speedy > 0 && m_engine.isFree(
				                   	getCenter().getX(),
				                   	getCenter().getY() + getRadius() + m_speedy,
				                   	getCenter().getZ(),
				                   	this
				           	)
				         ) 
				         ||
				         (m_speedy <= 0 && m_engine.isFree(
				                   	getCenter().getX(),
				                   	getCenter().getY() - getRadius() + m_speedy,
				                   	getCenter().getZ(),
				                   	this
				           	)
				         )
				       )
				    {
				        move(0, m_speedy, 0);
				        m_speedy += GameEngine.GRAVITY;
				    }
				    else
				        //Annars, stoppa yhastighet
				        m_speedy = 0;				
				}
				else
				{
					getCenter().setY( 0 );
					m_speedy = 0;
				}
			}
			//Under marken
			else
			if( ypos < 0 )
			{
			    //Stt p golvet igen
			    getCenter().setY( 0 );
			    m_speedy = 0;
			    
			}
		}
	}

	
	/**
	 * Stter ai:ns ml
	 * @param r nytt ml
	 */
	protected void setTarget(Figure r)
	{
		m_target = r;
	}
	
	/**
	 * Bestmmer om roboten ska tnka sjlv
	 * @param ai
	 */
	public void setAi(boolean ai)
	{
		m_isAi = ai;
	}
	
	/**
	 * Kollar om roboten tnker sjlv
	 * @return true om ai
	 */
	public boolean isAi()
	{
		return m_isAi;
	}
	
	/**
	 * Kollar om roboten lever
	 * @return true om roboten lever
	 */
	public boolean isAlive()
	{
		return m_isAlive;
	}

	/**
	 * Hmtar hlsan
	 * @return hlsa kvar
	 */
	public double getHealth()
	{
		return m_health;
	}
	
	/**
	 * Hmtar robotens namn
	 * @return robotens namn
	 */
	public String getName()
	{
		return m_name;
	}
	
	/**
	 * Hmtar hur mnga denna robot har ddat
	 * @return antal ddade
	 */
	public int getFrags()
	{
		return m_frags;
	}
	
	/**
	 * Hmtar hur mnga gnger denna robot blivit ddad
	 * @return antal gnger roboten har blivit ddad.
	 */	
	public int getKills()
	{
		return m_kills;
	}
	
	/**
	 * kar antalet ddade med 1.
	 */
	public void addFrag()
	{
		m_frags++;
	}

	/**
	 * Hmtar robotens id
	 * @return robotens id
	 */
	public int getId()
	{
		return m_id;
	}

	/**
	 * Hmtar robotens anvndarfrg
	 * @return anvndarfrg
	 */
	public Color getUserColor()
	{
		return m_userColor;
	}
	
	/**
	 * Fallhastighet
	 */
	protected double m_speedy=0;
	
	/**
	 * Om roboten lever
	 */
	protected boolean m_isAlive=true;
	
	/**
	 * Om roboten tnker sjlv
	 */
	protected boolean m_isAi=false;
	
	/**
	 * Antalet tick roboten har varit dd
	 */
	protected int m_deadTickCount=0;
	
	/**
	 * Antalet tick roboten har varit oddlig
	 */
	protected int m_immortalTickCount=0;
	
	/**
	 * Antalet tick sedan fregende skott
	 */
	protected int m_ammoDelayTickCount=0;
	
	/**
	 * Antalet tick som radarn har kvar att leva
	 */
	protected int m_radarTickCount=0;
	
	/**
	 * Hur mycket ammunition
	 */
	protected int m_ammo=Robot.START_AMMO;
	
	/**
	 * Hur mycket hlsa
	 */
	protected double m_health=Robot.START_HEALTH;
	
	/**
	 * Antalet tick sedan frra gngen roboten blev trffad.
	 */
	protected int m_lastHitTick=GameEngine.TPS;
	
	/**
	 * Den figur roboten jagar (gller endast fr ai)
	 */
	protected transient Figure m_target=null;
	
	/**
	 * Spelmotor som anvnds
	 */
	protected transient GameEngine m_engine=null;

	/**
	 * Anvndarfrg p roboten
	 */
	protected Color m_userColor;
	
	/**
	 * Namn p roboten
	 */
	protected String m_name;
	
	/**
	 * Antalet ddade
	 */
	protected int m_frags=0;
	
	/**
	 * Antalet gnger denna robot har blivit ddad
	 */
	protected int m_kills=0;
	
	/**
	 * Robotens id-nummer
	 */
	protected int m_id=0;
	
	/**
	 * "Gldande" frg
	 */
	protected Color m_glowColor=null;
	
	/**
	 * Den sol som glder
	 */
	protected Sun m_glowSun=null;
	
	/**
	 * Robottype
	 */
	protected int m_type=0;
	
	/**
	 * Rrelsehastighet p roboten
	 */
	public static final double MOVE_SPEED = 15.0/GameEngine.TPS;
	
	/**
	 * Fr trfftest. Vrde fr ingen trff
	 */
	public static final int NOHIT = 0;
	/**
	 * Fr trfftest. Vrde fr ddande trff
	 */
	public static final int KILLHIT = 1;
	/**
	 * Fr trfftest. Vrde fr skadande trff
	 */
	public static final int HURTHIT = 2;

	/**
	 * Antalet robottyper som det finns
	 */
	public static final int ROBOT_COUNT=2;
	/**
	 * Hur mnga tick det mste g mellan tv skott
	 */
	public static final int	AMMO_DELAY = GameEngine.TPS*20/100;
	
	/**
	 * Hlsa fr en nyfdd robot
	 */
    public static final double START_HEALTH = 100.0;
    /**
     * Maximal hlsa fr en robot
     */
    public static final double MAX_HEALTH = 200.0;
    /**
     * Antal skott fr en nyfdd robot
     */
	public static final int START_AMMO = 10;
	/**
	 * Maximalt antal skott
	 */
	public static final int MAX_AMMO = 50;
	/**
	 * Livslngden p radarn
	 */
    public static final int RADAR_LIFETIME = 7*GameEngine.TPS;
    /**
     * Hur lnge man kan vara oddlig
     */
    public static final int IMMORTAL_LIFETIME = 5*GameEngine.TPS;
    /**
     * Hur lnge roboten vrids nr det precis har dtt (tittar ned mot marken)
     */
	public static final int DEATH_ANIMATION = 3*GameEngine.TPS/10;
	/**
	 * Hur lngt man kan se med radarn.
	 */
	public static final double RADAR_VIEW_DISTANCE = 12;

	/**
	 * Hur mnga tick det mste g innan man kan teruppst
	 */
	protected static final int ticksBeforeRecarniation=GameEngine.TPS;
	
	/**
	 * Hastighet man har nr man hoppar
	 */
	protected static final double JUMP_SPEED = 23.3/GameEngine.TPS;
	
}

