import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;

import cubeengine.Camera;
import cubeengine.CubeEngine;
import cubeengine.Sun;



/**
 * En kontroller dr man vljer robot.
 * @author Per-Erik Kristensson
 */
public class RobotPickPanel extends JPanel implements MouseListener, Runnable
{
    /**
     * Skapar en robot panel
     * @param startColor Startfrg p roboten
     */
	public RobotPickPanel(Color startColor)
	{
		super();
		setPreferredSize( new Dimension(80, 80) );
		m_robot = new Robot(null, -1, "", startColor, 0);
		m_robot.setVisible(true);
		m_robot.rotateAxel(0, -Math.PI/20, 0);
		
		setVisible(true);
		addMouseListener(this);
		
		m_engine = new CubeEngine();
		m_engine.add(m_robot);
		
		Sun s = new Sun();
		s.setColor(Color.white);
		s.getCenter().set(0, 10, 10);
		s.setGamma(5);
		s.setVisible(false);
		m_engine.add(s);
				
		m_camera.getCenter().set(0, 0, 10);
		m_camera.setZoom(13.5);
		
		
		m_thread = new Thread(this);
		m_thread.start();
	}

	/**
	 * Kollar viken robottyp som r vald
	 * @return Robotyp
	 */
	public int getType()
	{
		return m_robotType;
	}

	/**
	 * Bestmmer vilken robottyp som ska anvndas
	 * @param type Ny robottyp
	 */
	synchronized public void setRobotType(int type)
	{
		m_robotType = type;
		m_robot.setRobotType(m_robotType);
		m_robot.rotateAxel(0, -Math.PI/20, 0);
	}
	
	/**
	 * Bestmmer vilken frg roboten ska ha
	 * @param color Ny robotfrg
	 */
	synchronized public void setUserColor(Color color)
	{
		m_robot.setUserColor(color);
		setRobotType(m_robotType);
	}
	
	/**
	 * Utritning
	 */
	public void paint(Graphics g1)
	{
		Graphics2D g = (Graphics2D) g1;

		Rectangle bound = getBounds();
	
		final Color backgroundcolor = new Color(0,0,20);
		g.setColor(backgroundcolor);
		g.fillRect(0, 0, bound.width, bound.height);
		
		m_engine.draw( g, bound.width/2, bound.height/2, m_camera );	
	}

	public void mouseClicked(MouseEvent arg0) 
	{
		setRobotType( (m_robotType+1)%Robot.ROBOT_COUNT );
	}
	public void mouseEntered(MouseEvent arg0) {}
	public void mouseExited(MouseEvent arg0) {}
	public void mousePressed(MouseEvent arg0) {}
	public void mouseReleased(MouseEvent arg0) {}

	public void run() 
	{
		while(m_run)
		{
		    //Rotera, rita om och pausa en stund
			m_robot.rotateAxel(Math.PI*2*0.35/GameEngine.TPS, 0, 0);
			repaint();

			try
			{
				Thread.sleep( 1000/GameEngine.TPS ); 
			}
			catch(InterruptedException e)
			{
				break;
			}
		}
	}
	
	/**
	 * Stannar roteringen
	 */
	public void stop()
	{
		m_run=false;
	}
	
	/**
	 * Robot som anvnds
	 */
	protected Robot m_robot;
	
	/**
	 * Trd som roterar roboten
	 */
	protected Thread m_thread;
	
	/**
	 * Grafikmotor
	 */
	protected CubeEngine m_engine;
	/**
	 * Kamera som tittar p roboten
	 */
	protected Camera m_camera = new Camera();
	
	/**
	 * Om trden ska fortstta
	 */
	protected boolean m_run=true;
	
	/**
	 * Vald robottyp
	 */
	protected int m_robotType = 0;
	

}