import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;


/**
 * Server tar hand som serverdelen i spelet. Den tar hand
 * om klienterna och initierar dom nr de ansluter till spelet.
 * 
 * @author Per-Erik Kristensson
 */
public class Server extends Thread implements ClientListener
{
	/**
	 * Skapar en server
	 */
	public Server()
	{
		super();

		try
		{
			m_socket = new ServerSocket(12345);
			System.out.println("Server is running...");
		}
		catch(IOException e)
		{
			System.out.println("Couldn't create socket");
			System.exit(1);
		}
	}

	/**
	 * Skickar ivg alla jobb till klienterna
	 * @param jobs Jobb som ska skickas
	 */
	public void sendJobs(ArrayList jobs)
	{
		for(int i=0; i<jobs.size(); i++)
			sendToClients( (Job) jobs.get(i) );

		sendToClients( new JobTick() );

		for(int c=0; c<m_clients.size(); c++)
			((Client) m_clients.get(c)).flush();
	}

	/**
	 * Anropas av klienterna nr spelet startar. Ignoreras
	 */
	public void startGame(Client sender)
	{}

	/**
	 * Stter vilken spelmotor som ska anvndas
	 * @param engine Spelmotor som ska anvndas
	 */
	public void setGameEngine(GameEngine engine)
	{
		m_engine = engine;
	}

	/**
	 * Kommer frn klienterna nr Job har tagits emot. 
	 * Om jobbet r av typen JobAddRobot s har klienten
	 * skickat vilken robot den vill anvnda. Den roboten
	 * lggs d till p servern och samtliga anslutna klienter.
	 * Drefter initieras en nyss anslutna klienten.
	 * 
	 * @param j Nytt job
	 * @param sender Klient som skickade meddelandet.
	 */
	public void jobRecieved(Job j, Client sender)
	{		
	    //Ska ny klient lggas till?
	    if(j instanceof JobAddRobot)
		{
	        JobAddRobot jadd = (JobAddRobot) j;
	        
	        int playercount = m_engine.getNumberOfRobots();
	        
	        //OK, gr det
	        m_engine.addJob(j);
	        	        
	        //Sedan, avvakta tills roboten har blivit tillagd.
	        while(m_engine.getNumberOfRobots() < playercount+1)
	        {
	            try
	            {
	                sleep(GameEngine.TPS*2);
	            }
	            catch(Exception e)
	            {};
	        }
        
	        //Stanna motorn en stund under initieringen
	        m_engine.stop();
	        
	        {
		        //Nu har roboten lagts till hos servern och alla klienter
		        //(utom den som nyss anslts).
		        System.out.println("Klient tillagd...");
		        
		        //Hitta vilket startid klienten.
		        int startId = m_engine.getNumberOfRobots()-1;

		        initClient(sender, startId);
	        }
	        
	        //Starta igen
	        m_engine.start();
	        
	        
		}
		//StartGame skickas till klienten nr spelet skall brja...
		else
		    m_engine.addJob(j);
	}
	
	/**
	 * Initiera en klient
	 * @param client Klient att initiera
	 * @param startId Vilket robotID klienten ska ha
	 */
	synchronized public void initClient(Client client, int startId)
	{
	    client.setEngine(m_engine);
	    client.setRobotId(startId);
		
		//Skicka startmeddelande
		Job j = new JobStartGame( 
		        		startId, //m_engine.getNumberOfRobots()-1,
		        		m_engine.m_robots,
		        		m_engine.m_ammos,
		        		m_engine.m_boxes
		        		);
		
		//sendToClient(client, j);
		client.send(j);
		client.flush();
		
		addClient(client);
	}
	
	/**
	 * Anropas nr en klient har kopplats ifrn. Den klienten
	 * kopplas bort frn spelet. Dess robot blir AI.
	 * 
	 * @param sender Klient som kopplade ifrn.
	 */
	public void clientDisconnected(Client sender)
	{
		removeClient(sender);
		
		if(sender.getRobotId() >= 0)
		{
			m_engine.getRobot( sender.getRobotId() ).setAi(true);
			System.out.println("ID " + sender.getRobotId() + " kopplade ifrn.");
		}
	}

	/**
	 * Kopplar bort alla klienter
	 */
	public void closeGame()
	{
		for(int i=0; i<m_clients.size(); i++)
			((Client)m_clients.get(i)).close();
	}

	/**
	 * Lgger till en klient.
	 * @param c klient som ska lggas till
	 */
	protected void addClient(Client c)
	{
		m_clients.add(c);
		System.out.println(m_clients.size() + " clients");
	}

	/**
	 * Tar bort en klient
	 * @param c klient som ska tas bort.
	 */
	protected void removeClient(Client c)
	{
		m_clients.remove(c);
		System.out.println(m_clients.size() + " clients");
	}

	/**
	 * Skickar ett jobb till samtliga klienter
	 * @param p Job som ska skickas
	 */
	public void sendToClients(Job p)
	{
		for(int i=0; i<m_clients.size(); i++)
			((Client) m_clients.get(i)).send( p );
	}
	
	/**
	 * Funktion fr trden som tar emot klienten
	 */
	public void run()
	{
		while(m_socket != null)
		{
			try
			{
				Socket client = m_socket.accept();
				Client c = new Client(client);	
				c.addClientListener(this);
				c.start();				
			}
			catch(IOException e)
			{
				System.out.println("server-run fel");
				System.exit(1);
			}
		}
	}

	/**
	 * Socketen som lyssnar p klienter
	 */
	protected ServerSocket m_socket=null;
	
	/**
	 * Array med alla anslutna klienter
	 */
	protected ArrayList m_clients = new ArrayList();
	
	/**
	 * Spelmotor som anvnds
	 */
	protected GameEngine m_engine=null;
}

