/*
 * Created on 2004-nov-25
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
package cubeengine;

import java.io.Serializable;

/**
 * En Camera anvnds fr att titta p 3d-vrlden.
 * En Camera r samtidigt en Figure, vilket kanske r lite
 * krystat, men r ganska praktiskt nr man gr spel.
 * 
 * @author Per-Erik Kristensson
 */
public class Camera extends Figure implements Serializable
{
    /**
     * Skapar en kamera
     */
	public Camera()
	{
		super("camera");
		createMatrix();
	}

	/**
	 * terstllet kameran
	 */
	public void reset()
	{
		m_vx=m_vy=m_vz=0;
		load("camera");
	}

	/**
	 * Skapar rotationsmatrisen som vrider punkterna vid uppritning
	 */
	public void createMatrix()
	{
		m_rotateMatrix = Cord3d.createRotateMatrix(m_vx, m_vy, m_vz);	
	}

	/**
	 * Retunerar kameran som en strng
	 */
	public String toString()
	{
		return "Camera: (" + getCenter().getX() + ", " + getCenter().getY() + ", " + getCenter().getZ() + ")  " +
			" (" + m_vx + ", " + m_vy + ", " + m_vz + ")  ";
	}

	/**
	 * Flyttar figuren relativt hur kameran r vinklad
	 * @param dx Skillnad i x-led
	 * @param dy Skillnad i y-led
	 * @param dz Skillnad i z-led
	 */
	public void relativeMove(double dx, double dy, double dz)
	{
		double[][] matrix = Cord3d.createRotateMatrix(m_vx, m_vy, -m_vz*0);	

		getCenter().add(   dx * matrix[0][0] + dy*matrix[0][1] + dz*matrix[0][2],
							dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2],
							dx * matrix[2][0] + dy*matrix[2][1] + dz*matrix[2][2]
			);
	}

	/**
	 * Roterar figuren relativt
	 * @param dx Vinkel i x-led
	 * @param dy Vinkel i y-led
	 * @param dz Vinkel i z-led
	 */
	public void relativeRotate(double dx, double dy, double dz)
	{
	    /*
	     * Denna funktion fungerar inte helt som det r tnkt.
	     * Man kan bara ndra blicken i x och y-led, skillnad
	     * i z ignoreras.
	     */
			    
	    double[][] matrix = Cord3d.createRotateMatrix(m_vx, m_vy, m_vz);

	    //Rotera tillbaka figuren (mste ske i tv steg).
	    rotateAxel(     -m_vx, 0, 0       );
	    rotateAxel(     0, -m_vy, 0       );
		
		m_vx -= dx;
		m_vy -= dx * matrix[1][0] + dy*matrix[1][1] + dz*matrix[1][2];

		//Rotera figuren till den nya platsen
		rotateAxel(     m_vx, m_vy, 0       );
                
		createMatrix();

	}

	/**
	 * Retunerar hur mycket en punkt ska frstoras. Om
	 * avstndet till punkten r samma som zoom-vrdet, 
	 * blir vrdet 1. r avstndet bara hlften, retuneras
	 * 2 och s vidare.
	 * 
	 * @param cord Punkt att rkna p
	 * @return Frstoringsfaktor
	 */
	public double getVisualResizeFactor(Cord3d cord)
	{
		//Om avstnder till punkten = zoom, blir frstoringsfaktorn 1
		//Om avstndet till punkten = zoom/2, blir frstoringsfaktorn 2....
		//S f=zoom/avstnd

		//D punkter framfr kameran har negativ z-kordinat, invertera
		return -m_zoom/getCameraZ(cord);
	}

	/**
	 * Avbildar en punkt p skrmen.
	 * @param cord Punkt att avbilda
	 * @param centerx Mittpunkt i x-led p skrmen
	 * @return Vilken x-kordinat punkten ska ha p skrmen
	 */
	public int getVisualXPos(Cord3d cord, int centerx)
	{
		double factor = getVisualResizeFactor(cord);
		
		if(getCameraZ(cord) >= 0)
		    return centerx - (int)(m_size * factor * getCameraX(cord) +0.5);

		return centerx + (int)(m_size * factor * getCameraX(cord) +0.5);
	}
	
	/**
	 * Avbildar en punkt p skrmen.
	 * @param cord Punkt att avbilda
	 * @param centery Mittpunkt i y-led p skrmen
	 * @return Vilken y-kordinat punkten ska ha p skrmen
	 */
	public int getVisualYPos(Cord3d cord,int centery)
	{
		double factor = getVisualResizeFactor(cord);

		if(getCameraZ(cord) >= 0)
		    return centery + (int)(m_size * factor * getCameraY(cord) +0.5);

		return centery - (int)(m_size * factor * getCameraY(cord) +0.5);
	} 


	/**
	 * Omvandlar en punkt s att den r relativ kameran
	 * @param cord Punkt att omvandla
	 * @return en punkt relativ kameran
	 */
	public Cord3d getCameraCord(Cord3d cord)
	{
		return new Cord3d(	getCameraX(cord),
			getCameraY(cord),
			getCameraZ(cord));
	}

	/**
	 * Rknar ut vilken punkt en punkt har relativt kameran
	 * @param cord punkt att rkna p
	 * @return Det relativa vrdet i x-led fr punkten
	 */
	public double getCameraX(Cord3d cord)
	{
		return	(cord.getX()-getCenter().getX()) * m_rotateMatrix[0][0] +
			(cord.getY()-getCenter().getY()) * m_rotateMatrix[1][0] + 
			(cord.getZ()-getCenter().getZ()) * m_rotateMatrix[2][0];
	}

	/**
	 * Rknar ut vilken punkt en punkt har relativt kameran
	 * @param cord punkt att rkna p
	 * @return Det relativa vrdet i y-led fr punkten
	 */
	public double getCameraY(Cord3d cord)
	{
		return	(cord.getX()-getCenter().getX()) * m_rotateMatrix[0][1] +
			(cord.getY()-getCenter().getY()) * m_rotateMatrix[1][1] + 
			(cord.getZ()-getCenter().getZ()) * m_rotateMatrix[2][1];
	}

	/**
	 * Rknar ut vilken punkt en punkt har relativt kamerant
	 * @param cord punkt att rkna p
	 * @return Det relativa vrdet i z-led fr punkten
	 */
	public double getCameraZ(Cord3d cord)
	{
		return	(cord.getX()-getCenter().getX()) * m_rotateMatrix[0][2] +
			(cord.getY()-getCenter().getY()) * m_rotateMatrix[1][2] + 
			(cord.getZ()-getCenter().getZ()) * m_rotateMatrix[2][2];
	}

	/**
	 * Retunerar kamerans vinkel i x-led
	 * @return Vinkel i x-led
	 */
	public double getAngleX()
	{
		return m_vx;
	}
	
	/**
	 * Retunerar kamerans vinkel i y-led
	 * @return Vinkel i y-led
	 */
	public double getAngleY()
	{
		return m_vy;
	}
	
	/**
	 * Retunerar kamerans vinkel i z-led
	 * @return Vinkel i z-led
	 */
	public double getAngleZ()
	{
		return m_vz;
	}

	/**
	 * Hmtar kamerans zoomvrde
	 * @return Kamerans zoomvrde
	 */
	public double getZoom()
	{
		return m_zoom;
	}

	/**
	 * Hmtar kamerans normalvrde
	 * @return Kamerans normalvrde
	 */
	public double getNormalSize()
	{
		return m_size;
	}

	/**
	 * Stter kamerans zoomvrde
	 * @param zoom Nytt vrde
	 */
	public void setZoom(double zoom)
	{
		m_zoom=zoom;
	}

	/**
	 * Stter kamerans normalvrde. Normalvrdet r hur
	 * mnga punkter en strcka har om den befinner sig p samma
	 * avstnd som zoomen r satt till.
	 * 
	 * @param size Nytt vrde
	 */
	public void setNormalSize(double size)
	{
		m_size=size; 
	}

	/**
	 * De vinklar som kameran r vriden i
	 */
	protected double m_vx=0,m_vy=0,m_vz=0;
	
	/**
	 * Rotationsmatris som vrider vrlden vid uppritning
	 */
	protected double[][] m_rotateMatrix;
	
	/**
	 * Zoom och normallngd.
	 */
	protected double m_zoom = 10, m_size = 40;
}

