package cubeengine;

import java.io.Serializable;
import java.util.ArrayList;

/**
 * Cord3d r en kordinat i rummet,
 * har ett vrde i x, y och z-led.
 * 
 * Punkten kan flyttas och roteras p olika stt.
 * 
 * @author PEK
 */
public final class Cord3d implements Comparable, Cloneable, Serializable
{
	/**
	 * Skapa ett Cord3d-objekt i origo
	 */
    public Cord3d()
	{m_x=m_y=m_z=0;};


	/**
	 * Skapar ett Cord3d-objekt i (x,y,z)
	 * @param x	X-kordinat
	 * @param y Y-kordinat
	 * @param z Z-kordinat
	 */
	public Cord3d(double x, double y, double z)
	{
		m_x=x;
		m_y=y;
		m_z=z;
	}

	/**
	 * Skapar en kopia av ett Cord3d-objekt
	 * @param o  Objekt som kopieras
	 */
	public Cord3d(Cord3d o)
	{
		m_x=o.m_x;
		m_y=o.m_y;
		m_z=o.m_z;
	}

	/**
	 * Klonar
	 */
	public Object clone()
	{
		return new Cord3d(this);
	}

	/**
	 * 
	 * @return x-kordinaten
	 */
	public double getX()
	{
		return m_x;
	}

	/**
	 * 
	 * @return y-kordinaten
	 */
	public double getY()
	{
		return m_y;
	}

	/**
	 * 
	 * @return z-kordinaten
	 */
	public double getZ()
	{
		return m_z;
	}

	/**
	 * ndrar x-kordinaten
	 * @param x Ny x-position
	 */
	public void setX(double x)
	{
		m_x=x;
	}
	
	/**
	 * ndrar y-kordinaten
	 * @param y Ny y-position
	 */
	public void setY(double y)
	{
		m_y=y;
	}

	/**
	 * ndrar z-kordinaten
	 * @param z Ny z-position
	 */
	public void setZ(double z)
	{
		m_z=z;
	}

	/**
	 * Berknar summan av tv kordinaten
	 * @param a Kordinat 1
	 * @param b Kordinat 2 
	 */
	public static Cord3d sum(Cord3d a, Cord3d b)
	{
		return new Cord3d(	a.m_x + b.m_x,
			a.m_y + b.m_y,
			a.m_z + b.m_z );
	}

	/**
	 * Frndrar denna kordinats vrden
	 * @param a Kordinat att addera
	 */
	public void add(Cord3d a)
	{
		m_x +=a.m_x;
		m_y +=a.m_y;
		m_z +=a.m_z;
	}

	/**
	 * kar denna kordinats vrden
	 * @param x Frndring i x-led
	 * @param y Frndring i y-led
	 * @param z Frndring i z-led
	 */
	public void add(double x, double y, double z)
	{
		m_x +=x;
		m_y +=y;
		m_z +=z;
	}

	/**
	 * Minskar denna kordinats vrden
	 * @param a Kordinat med de vrden som ska dras ifrn
	 */
	public void subtract(Cord3d a)
	{
		m_x -=a.m_x;
		m_y -=a.m_y;
		m_z -=a.m_z;
	}

	/**
	 * Stter ny position
	 * @param x Nytt x-vrde
	 * @param y Nytt y-vrde
	 * @param z Nytt z-vrde
	 */
	public void set(double x, double y, double z)
	{
		m_x = x;
		m_y = y;
		m_z = z;
	}

	/**
	 * Stter ny position
	 * @param c Objekt att kopiera
	 */
	public void set(Cord3d c)
	{
		m_x = c.m_x;
		m_y = c.m_y;
		m_z = c.m_z;
	}

	/**
	 * Retunerar meddelvrdet av en array
	 * @param v  ArrayList med Cord3d-objekt
	 * @return Medelvrdet av alla kordinater
	 */
	public static Cord3d getCenter(ArrayList v)
	{
		return new Cord3d( getCenterX(v), getCenterY(v), getCenterZ(v) );
	}

	/**
	 * Retunerar meddelvrdet i x-led av en array
	 * @param obj  ArrayList med Cord3d-objekt
	 * @return Medelvrdet av alla kordinater i x-led
	 */
	public static double getCenterX(ArrayList obj)
	{
		double sum=0; 

		for(int i=0; i<obj.size(); i++)
			sum += ( (Cord3d) obj.get(i)).m_x;

		return sum/obj.size();
	}

	/**
	 * Retunerar meddelvrdet i y-led av en array
	 * @param obj  ArrayList med Cord3d-objekt
	 * @return Medelvrdet av alla kordinater i y-led
	 */
	public static double getCenterY(ArrayList obj)
	{
		double sum=0; 

		for(int i=0; i<obj.size(); i++)
			sum += ( (Cord3d) obj.get(i)).m_y;

		return sum/obj.size();
	}

	/**
	 * Retunerar meddelvrdet i z-led av en array
	 * @param obj  ArrayList med Cord3d-objekt
	 * @return Medelvrdet av alla kordinater i z-led
	 */
	public static double getCenterZ(ArrayList obj)
	{
		double sum=0; 

		for(int i=0; i<obj.size(); i++)
			sum += ( (Cord3d) obj.get(i)).m_z;

		return sum/obj.size();
	}

	/**
	 * Jmfr tv Cord3d-objekt.
	 * Om detta objekt ligger bakom objekt o
	 * (har lgre z-vrde) retuneras 1.
	 * Annars retuneras -1
	 */
	public int compareTo(Object o) 
	{
		if( ((Cord3d)o).m_z < m_z )
			return 1;

		return -1;                
	}    


	/**
	 * Roterar denna kordinat runt en viss punkt
	 * @param matrix - Roteringspunkt
	 * @param px  Punkt i x-led
	 * @param py  Punkt i y-led
	 * @param pz  Punkt i z-led
	 */
	public void rotate(double[][] matrix, double px, double py, double pz)
	{       
		m_x -= px;
		m_y -= py;
		m_z -= pz;

		double newx = m_x * matrix[0][0] + m_y*matrix[0][1] + m_z*matrix[0][2];
		double newy = m_x * matrix[1][0] + m_y*matrix[1][1] + m_z*matrix[1][2];
		double newz = m_x * matrix[2][0] + m_y*matrix[2][1] + m_z*matrix[2][2];

		m_x=newx+px;
		m_y=newy+py;
		m_z=newz+pz;
	}



	/**
	 * Roterar denna kordinat runt en viss punkt
	 * @param vx  Vinkel i x-led
	 * @param vy  Vinkel i y-led
	 * @param vz  Vinkel i z-led
	 * @param px  Punkt i x-led
	 * @param py  Punkt i y-led
	 * @param pz  Punkt i z-led
	 */
	public void rotate(double vx, double vy, double vz, double px, double py, double pz)
	{
		rotate( createRotateMatrix(vx, vy, vz), px, py, pz );
	}

	/**
	 * Roterar denna kordinat runt en viss punkt
	 * @param vx  Vinkel i x-led
	 * @param vy  Vinkel i y-led
	 * @param vz  Vinkel i z-led
	 * @param obj  Kordinat att rotera runt
	 */
	public void rotate(double vx, double vy, double vz, Cord3d obj)
	{
		rotate(vx, vy, vz, obj.m_x, obj.m_y, obj.m_z);
	}

	/**
	 * Roterar denna kordinat runt origo
	 * @param vx  Vinkel i x-led
	 * @param vy  Vinkel i y-led
	 * @param vz  Vinkel i z-led
	 */
	public void rotate(double vx, double vy, double vz)
	{
		rotate(vx,vy,vz, 0.0, 0.0, 0.0);
	}

	/**
	 * Roterar denna kordinat runt origo
	 * @param matrix  Roteringsmatris
	 */
	public void rotate(double[][] matrix)
	{
		rotate(matrix, 0.0, 0.0, 0.0);
	}


	/**
	 * Skapar en rotationsmatris. Om jag minns rtt s roteras frst
	 * z-vrdet, sedan y och sedan x.
	 * 
	 * @param vx  Vinkel i x-led
	 * @param vy  Vinkel i y-led
	 * @param vz  Vinkel y z-led
	 * @return roteringsmatris
	 */
	public static double[][] createRotateMatrix(double vx, double vy, double vz)
	{

		/*double 	ca = Math.cos(vz),
				cb = Math.cos(vx),
				cc = Math.cos(vy),
				sa = Math.sin(vz),
				sb = Math.sin(vx),
				sc = Math.sin(vy);*/
	    
	    //Anta att vinkeln r noll fr alla, rkna om om ndvndigt.
	    //r lite snabbare, men ingen strre skillnad.
	    double 	ca = 1,
				cb = 1,
				cc = 1,
				sa = 0,
				sb = 0,
				sc = 0;
	    
	    if(vz != 0)
	    {
	        ca = Math.cos(vz);
	        sa = Math.sin(vz);
	    }
	    
	    if(vx != 0)
	    {
	        cb = Math.cos(vx);
	        sb = Math.sin(vx);
	    }
	    
	    if(vy != 0)
	    {
	        cc = Math.cos(vy);
	        sc = Math.sin(vy);
	    }

		//Konstruera rotationsmotrisen, en grotesk liten skapelse.
		return new double[][]
			{ 
				{ ca*cb,	-sa*cc-ca*sb*sc,	sa*sc-ca*sb*cc },
				{ sa*cb,	ca*cc-sa*sb*sc,		-ca*sc-sa*sb*cc },
				{ sb,		cb*sc,				cb*cc} 
			};

	}

	/**
	 * Retunerar avstndet till en punkt
	 * @param x
	 * @param y
	 * @param z
	 * @return Avstndet till punkten (x,y,z)
	 */
	public double getDistance(double x, double y, double z)
	{
		return Math.sqrt(	(m_x-x)*(m_x-x)+
			(m_y-y)*(m_y-y)+
			(m_z-z)*(m_z-z)
			);
	}

	/**
	 * Retunerar avstnder till punkten c
	 * @param c  Punkt att bestmma avstnd till
	 * @return Avstnd mellan denna och c.
	 */
	public double getDistance(Cord3d c)
	{
		return getDistance(c.m_x, c.m_y, c.m_z);
	}

	/**
	 * Retunerar avstndet till en figur
	 * @param f  Figur att avstndsbestmma
	 * @return Avstndet till figuren f
	 */
	public double getDistance(Figure f)
	{
		return getDistance(f.getCenter().getX(), f.getCenter().getY(), f.getCenter().getZ() );
	}

	/**
	 * Retunerar objektet som en strng
	 */
	public String toString()
	{
		return "x: " + m_x + " y: " + m_y + " z: " + m_z;
	}

	/**
	 * Kordinater
	 */
	private double m_x, m_y, m_z; 

	/**
	 * Cachning. Anvnds fr att slippa rkna om pixel-
	 * kordinaten fr punkten. Borde nog
	 * ligga i en subklass. 
	 */

	int m_cache_visualx,
		m_cache_visualy;
}
