package cubeengine;

import java.awt.Graphics2D;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * CubeEngine r en simpel grafikmotor.
 * Man kan lgga in figurer och solar,
 * och sedan rendera en bild med draw.
 * 
 * @author PEK
 *
 */
public class CubeEngine
{
	/**
	 * Skapar ett CubeEngine objekt
	 *
	 */
    public CubeEngine() {};

	/**
	 * Lgger till figurer till CubeEngine.
	 * 
	 * @param figures Lista p figurer
	 */
	synchronized public void add(Figure[] figures)
	{
		for(int i=0; i<figures.length; i++)
		{
			add(figures[i]);
		}
	}

	/**
	 * Lgger till en figur i CubeEngine
	 * 
	 * @param figure Figur som ska lggas till
	 */	
	synchronized public void add(Figure figure)
	{
		m_all.add( figure );

		if(figure instanceof Sun)
			m_suns.add( figure );
		else
			m_figures.add( figure );
	}

	/**
	 * Tar bort samtliga figurer.
	 */
	public void clear()
	{
		m_suns.clear();
		m_all.clear();
		m_figures.clear();
	}

	/**
	 * Tar bort en figur
	 * @param f Figure som ska tas bort
	 */
	public synchronized void remove(Object f)
	{
		m_suns.remove(f);
		m_all.remove(f);
		m_figures.remove(f);
	}

	/**
	 * Hmtar antalet figurer
	 * @return Retunerar antalet figurer
	 */

	public int getFigureCount()
	{
		return m_all.size();
	}

	/**
	 * Hmta en figur
	 * @param n Nummer p den figur som ska retuneras
	 * @return Retunerar figur nummer n
	 */
	public Figure getFigure(int n)
	{
		return (Figure) m_all.get(n);
	}

	/**
	 * Renderar alla figurer. centerx och centery antas
	 * vara hlften av hjden/bredden
	 *  
	 * @param g			Objekt som det ska ritas i
	 * @param centerx	Centrum i x-led
	 * @param centery	Centrum i y-led
	 * @param camera	Kamera som bestmmer perspektivet
	 */
	public synchronized void draw(Graphics2D g, int centerx, int centery, Camera camera)
	{
		Sun[] suns = new Sun[m_suns.size()];
		m_suns.toArray(suns);

		class FigureComparer implements Comparator 
		{
			FigureComparer(Camera c)
			{
				m_camera = c;
			}

			public int compare(Object obj1, Object obj2)
			{
				Figure f1 = (Figure) obj1;
				Figure f2 = (Figure) obj2;

				if( m_camera.getCameraZ( f1.getCenter() ) > m_camera.getCameraZ( f2.getCenter() ) )
					return 1;

				return -1;
			}

			Camera m_camera;
		}

		//Sorterar alla objekt. Egenligen ondigt, det hade varit bttre
		//att plocka fram de synliga objekten, och bara sortera dessa
		Collections.sort( m_all, new FigureComparer(camera) );

		for(int i=0; i<m_all.size(); i++)
		{
			
			
			Figure o = (Figure) m_all.get(i);

			//Saker framfr kameran har negativ z-kordinat.
			if(camera.getCameraZ(o.getCenter()) >= 0) //-4)
				break;
			
			
			
			if(!o.isVisible())
				continue;

			
			
			o.draw(g, centerx, centery, suns, camera);                        
		}
	}

	private ArrayList
		/**
		 * Array med alla solar
		 */
		m_suns = new ArrayList(),
		/**
		 * Array med alla figurer
		 */
		m_figures = new ArrayList(),
		/**
		 * Array med alla objekt
		 */
		m_all = new ArrayList();
	
}