package cubeengine;

import java.awt.Color;
import java.awt.Graphics2D;
import java.io.Serializable;


/**
 * En sun agerar som en lampa som lyser upp andra figurer.
 * En sun ritas sjlv ut som en cirkel.
 * 
 * @author Per-Erik Kristensson
 */
public class Sun extends Figure implements Serializable
{
    /**
     * Skapar en Sun
     */ 
	public Sun()
	{ 
	    super (); 
	}
	
	/**
	 * Skapar en Sun i punkten (x,y,z), med frgen color
	 * @param x Plats i x-led
	 * @param y Plats i y-led
	 * @param z Plats i z-ed
	 * @param color Frg
	 */
	public Sun(double x, double y, double z, Color color)
	{
		super();

		//m_radius = 1;
		setRadius(0.31);

		getCenter().set(x, y, z);

		m_color=color;
	}
	
	/**
	 * Stter frgen
	 * @param color Ny frg
	 */
	public void setColor(Color color)
	{
		m_color = color;
	}

	/**
	 * Ritar ut solen
	 */
	public void draw(  Graphics2D g, 
		int centerx, 
		int centery, 
		Sun[] sun, 
		Camera camera
		)
	{
		double radius = getRadius()*camera.getVisualResizeFactor(getCenter())*camera.getNormalSize();

		int x = camera.getVisualXPos(getCenter(), centerx),
			y = camera.getVisualYPos(getCenter(), centery);


		if( x - radius <= 2*centerx &&
			x+radius >= 0			&&
			y - radius <= 2*centery &&
			y+radius >=0 )
		{	
			drawSetTransparency(g);

			g.setColor(m_color);
			g.fillOval( (int)(x - radius),
				(int)(y - radius),
				(int)(radius*2), 
				(int)(radius*2));

			drawResetTransparency(g);
		}
	}
	
	/**
	 * Retunerar solens frg
	 * @return Solens frg
	 */
	public Color getColor()
	{
		return m_color;
	}
	
	/**
	 * Skapar en color utifrn vrden i r,g,b.
	 * R,g,b behver inte ligga innanfr 0-255
	 * 
	 * @param r
	 * @param g
	 * @param b
	 * @return Frgen
	 */
	public static Color getColor(int r, int g, int b)
	{
		if(r<0) r=0;
		if(r>255) r=255;
		if(g<0) g=0;
		if(g>255) g=255;
		if(b<0) b=0;
		if(b>255) b=255;

		return new Color(r,g,b);
	}

	/**
	 * Stter solen "gamma"-vrde. Allts hur ljusstark solen r.
	 * @return gammavrdet
	 */
	public double getGamma()
	{
		return m_gamma;
	}
	
	/**
	 * Stter solens gamma-vrde 
	 * @param gamma nytt vrde
	 */
	public void setGamma(double gamma)
	{
		m_gamma = gamma;
	}
	
	/**
	 * Solens ljusstyrka
	 */
	protected double m_gamma = 2;
	
	/**
	 * Solens frg
	 */
	protected Color m_color;
}
